package com.abssqr.plat.biz.api.astPool;

import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.dal.mysql.auto.paging.AstPoolPage;
import com.abssqr.plat.common.facade.base.BaseStringParam;
import com.abssqr.plat.common.facade.enums.AstTypeEnum;
import com.abssqr.plat.common.facade.param.astPool.AstPoolOprParam;
import com.abssqr.plat.common.facade.param.astPool.AstPoolPageQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.astPool.AstPoolVO;
import com.abssqr.plat.common.facade.service.AstPoolService;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.convertor.AstPoolConvert;
import com.abssqr.plat.common.model.domain.ast.AstPoolEntity;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.repo.astPool.AstPoolRepository;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.core.service.ast.AstPackManager;
import com.general.system.common.util.VarChecker;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class AstPoolServiceImpl extends ServiceSupport implements AstPoolService{

    @Autowired
    AstPoolRepository astPoolRepository;

    @Autowired
    AstPackManager astPackManager;

    /**
     * 新增资产池
     * @param astPoolOprParam
     * @return
     */
    @Override
    public int add(@Valid AstPoolOprParam astPoolOprParam) {
        return super.doOpr(astPoolOprParam,()->{
            AstPoolEntity astPoolEntity=new AstPoolEntity();
            astPoolEntity.setAstTypeEnum(AstTypeEnum.getByCode(astPoolOprParam.getAstType()));
            //如果前端传入产品编号为空则将资产类型编号作为产品编号
            if(StringUtils.isEmpty(astPoolEntity.getProdNo())){
                astPoolEntity.setProdNo(astPoolOprParam.getAstType());
            }
            astPoolEntity.setAstPoolNo(SequenceUtil.genId(IdTypeEnum.AST_POOL_CODE));
            astPoolEntity.setAstPoolName(astPoolOprParam.getAstPoolName());
            VarChecker.checkArgument(astPoolRepository.checkAstPool(astPoolEntity),"资产池名称/资产池编号重复");

            //创建默认资产包
            astPackManager.craeteDefaultPack(astPoolEntity.getAstPoolNo(),astPoolEntity.getAstTypeEnum(),astPoolEntity.getProdNo());
            return astPoolRepository.create(astPoolEntity);
        });
    }

    /**
     * 分页查询资产池列表
     * @param astPoolPageQryParam
     * @return
     */
    @Override
    public BasePage<AstPoolVO> page(@Valid AstPoolPageQryParam astPoolPageQryParam) {
        return super.doQry(astPoolPageQryParam,()->{
            BasePage<AstPoolVO> result=new BasePage();
            AstPoolPage page = astPoolRepository.getPage(astPoolPageQryParam);
            result.setDatas(AstPoolConvert.convert2VOs(page.getDatas()));
            result.setCurrPageNo(page.getCurrPageNo());
            result.setTotal(page.getTotal());
            return result;
        });

    }

    /**
     * 查询资产池详情
     * @param astPoolNo
     * @return
     */
    @Override
    public AstPoolVO get(String astPoolNo) {
        return super.doQry(new BaseStringParam(astPoolNo),()->{
            AstPoolEntity astPoolEntity = astPoolRepository.getByAstPoolNo(astPoolNo);
            VarChecker.checkNotNull(astPoolEntity,"资产池不存在");
            return AstPoolConvert.convert2VO(astPoolEntity);
        });
    }

}
