/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.biz.validate.validator;

import com.abssqr.plat.biz.validate.AFieldValidator;
import com.abssqr.plat.biz.validate.FieldError;
import com.abssqr.plat.biz.validate.ValidateService;
import com.abssqr.plat.common.facade.validation.VldString;
import org.apache.commons.lang3.StringUtils;

/**
 *
 * @author zhenxuan.luo
 * @version $Id: StringValidator.java, v 0.1 2015年5月19日 上午11:04:04 zhenxuan.luo Exp $
 */
public final class StringValidator extends AFieldValidator<VldString> {
    
    /**
     * 默认构造器
     */
    public StringValidator() {
        super(VldString.class);
    }
    
    /**
     * @param annotationClass
     */
    public StringValidator(Class<VldString> annotationClass) {
        super(annotationClass);
    }
    
    @Override
    public boolean isValid(ValidateService validateService, VldString annotationObj,
                           Object validTargetFieldObj, FieldError errorConext) {
        boolean isValid = true;
        try {
            return isValid = validate(annotationObj, validTargetFieldObj);
        } finally {
            if (!isValid) {
                errorConext.setErrorMsg(annotationObj.message());
            }
        }
    }
    
    /**
     * 实际校验逻辑
     *
     * @param annotationObj
     * @param validTargetFieldObj
     * @return
     */
    private boolean validate(VldString annotationObj, Object validTargetFieldObj) {
    
        if (null == validTargetFieldObj) {
            return (annotationObj.nullable()) ? true : false;
        }
    
        if (validTargetFieldObj instanceof String) {
            String checkText = (String) validTargetFieldObj;
            boolean ret = checkEmpty(annotationObj, checkText);
        
            // 空校验通过，且需要校验range
            if (ret) {
                return (checkText.length() >= annotationObj.minLength())
                       && (checkText.length() <= annotationObj.maxLength());
            }
        
            return ret;
        }
    
        return false;
    }
    
    /**
     * blank校验
     *
     * @return
     */
    private boolean checkEmpty(VldString annotationObj, String checkText) {
    
        if (annotationObj.checkEmpty()) {
            return !StringUtils.isBlank(checkText);
        } else {
            return true;
        }
    }
}
