package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstPoolDO;
import java.util.List;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.paging.AstPoolPage;
import com.abssqr.plat.common.dal.mysql.auto.mapper.AstPoolDOMapper;

/**
* The Table abs_ast_pool.
* ABS_AST_POOL
*/
@Repository
public class AstPoolDAO{

    @Autowired
    private AstPoolDOMapper astPoolDOMapper;

    /**
     * desc:插入表:abs_ast_pool.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(AstPoolDO entity){
        return astPoolDOMapper.insert(entity);
    }
    /**
     * desc:批量插入表:abs_ast_pool.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<AstPoolDO> list){
        astPoolDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:查询是否有重复资产池名称和资产池编码的资产池.<br/>
     * @param prodNo prodNo
     * @param astPoolName astPoolName
     * @return Long
     */
    
    public Long getDupliCnt(String prodNo,String astPoolName){
        return astPoolDOMapper.getDupliCnt(prodNo, astPoolName);
    }
    /**
     * desc:分页查找资产池.<br/>
     * @param astPool astPool
     * @return AstPoolPage
     */
    
    public AstPoolPage getPage(AstPoolPage astPool){
        int total = astPoolDOMapper.getPageCount(astPool);
        if(total>0){
            astPool.setDatas(astPoolDOMapper.getPageResult(astPool));
        }else{
            astPool.setDatas(new ArrayList());
        }
        astPool.setTotal(total);
        return astPool;
    }
    /**
     * desc:根据资产池编号查找资产池.<br/>
     * @param astPoolNo astPoolNo
     * @return AstPoolDO
     */
    
    public AstPoolDO getByNo(String astPoolNo){
        return astPoolDOMapper.getByNo(astPoolNo);
    }
    /**
     * desc:根据主键删除数据:abs_ast_pool.<br/>
     * @param id id
     * @return int
     */
    
    public int deleteById(Long id){
        return astPoolDOMapper.deleteById(id);
    }
    /**
     * desc:根据主键获取数据:abs_ast_pool.<br/>
     * @param id id
     * @return AstPoolDO
     */
    
    public AstPoolDO getById(Long id){
        return astPoolDOMapper.getById(id);
    }
}
