package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanAcctTitleLogDO;
import java.util.List;
import java.util.Date;
import com.abssqr.plat.common.dal.mysql.auto.mapper.PlanAcctTitleLogDOMapper;

/**
* The Table abs_plan_acct_title_log.
* 计划结算科目信息（收付实现制）
*/
@Repository
public class PlanAcctTitleLogDAO{

    @Autowired
    private PlanAcctTitleLogDOMapper planAcctTitleLogDOMapper;

    /**
     * desc:批量插入表:abs_plan_acct_title_log.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<PlanAcctTitleLogDO> list){
        planAcctTitleLogDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:根据主键获取数据:abs_plan_acct_title_log.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param planNo planNo
     * @return List<PlanAcctTitleLogDO>
     */
    
    public List<PlanAcctTitleLogDO> getByDate(Date endDate,Date startDate,String planNo){
        return planAcctTitleLogDOMapper.getByDate(endDate, startDate, planNo);
    }
    /**
     * desc:根据主键获取数据:abs_plan_acct_title_log.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param calcType calcType
     * @param acctEvent acctEvent
     * @return PlanAcctTitleLogDO
     */
    
    public PlanAcctTitleLogDO getTitleLogByCondition(Date accountDate,String planNo,String calcType,String acctEvent){
        return planAcctTitleLogDOMapper.getTitleLogByCondition(accountDate, planNo, calcType, acctEvent);
    }
    /**
     * desc:汇总统计会计日对应科目流水金额.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param acctEvent acctEvent
     * @param listTitleType listTitleType
     * @return PlanAcctTitleLogDO
     */
    
    public PlanAcctTitleLogDO getSummaryAcctTitleByTypes(Date accountDate,String planNo,String acctEvent,List<String> listTitleType){
    if(listTitleType!=null && listTitleType.size()>201){throw new RuntimeException("listTitleType 超出201长度限制");}
        return planAcctTitleLogDOMapper.getSummaryAcctTitleByTypes(accountDate, planNo, acctEvent, listTitleType);
    }
    /**
     * desc:根据计划事件码科目编号获取科目流水.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param titleNo titleNo
     * @param acctEvent acctEvent
     * @return List<PlanAcctTitleLogDO>
     */
    
    public List<PlanAcctTitleLogDO> getByEventAndTitleNo(Date accountDate,String planNo,String titleNo,String acctEvent){
        return planAcctTitleLogDOMapper.getByEventAndTitleNo(accountDate, planNo, titleNo, acctEvent);
    }
    /**
     * desc:根据计划与事件码获取科目流水.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param acctEvent acctEvent
     * @param listTitleType listTitleType
     * @return List<PlanAcctTitleLogDO>
     */
    
    public List<PlanAcctTitleLogDO> getByPlanAndEvent(Date accountDate,String planNo,String acctEvent,List<String> listTitleType){
    if(listTitleType!=null && listTitleType.size()>201){throw new RuntimeException("listTitleType 超出201长度限制");}
        return planAcctTitleLogDOMapper.getByPlanAndEvent(accountDate, planNo, acctEvent, listTitleType);
    }
}
