package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.TrTaskDO;
import java.util.List;
import java.util.Date;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.TransferFailTaskStat;
import com.abssqr.plat.common.dal.mysql.auto.mapper.TrTaskDOMapper;

/**
* The Table ABS_TR_TASK.
* 转让任务
*/
@Repository
public class TrTaskDAO{

    @Autowired
    private TrTaskDOMapper trTaskDOMapper;

    /**
     * desc:插入表:ABS_TR_TASK.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(TrTaskDO entity){
        return trTaskDOMapper.insert(entity);
    }
    /**
     * desc:批量插入表:ABS_TR_TASK.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<TrTaskDO> list){
        trTaskDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:根据主键删除数据:ABS_TR_TASK.<br/>
     * @param id id
     * @return int
     */
    
    public int deleteById(Long id){
        return trTaskDOMapper.deleteById(id);
    }
    /**
     * desc:根据主键获取数据:ABS_TR_TASK.<br/>
     * @param id id
     * @return TrTaskDO
     */
    
    public TrTaskDO getById(Long id){
        return trTaskDOMapper.getById(id);
    }
    /**
     * desc:获取初始化状态的任务.<br/>
     * @param curTime curTime
     * @param limit limit
     * @param batchNo batchNo
     * @param packType packType
     * @return List<TrTaskDO>
     */
    
    public List<TrTaskDO> selectInitTask(Date curTime,Long limit,String batchNo,String packType){
        return trTaskDOMapper.selectInitTask(curTime, limit, batchNo, packType);
    }
    /**
     * desc:根据主键对数据加锁：ABS_TR_TASK.<br/>
     * @param id id
     * @return TrTaskDO
     */
    
    public TrTaskDO lockByIdNowait(Long id){
        return trTaskDOMapper.lockByIdNowait(id);
    }
    /**
     * desc:更新执行的任务状态.<br/>
     * @param taskId taskId
     * @param memo memo
     * @param finishStatus finishStatus
     * @return int
     */
    
    public int updateTaskStatus(Long taskId,String memo,String finishStatus){
        return trTaskDOMapper.updateTaskStatus(taskId, memo, finishStatus);
    }
    /**
     * desc:筛选完成后将此批次数据重置为初始化状态.<br/>
     * @param batchNo batchNo
     * @param taskStatus taskStatus
     * @param oldTaskStatus oldTaskStatus
     * @return int
     */
    
    public int updateBatchTaskStatus(String batchNo,String taskStatus,String oldTaskStatus){
        return trTaskDOMapper.updateBatchTaskStatus(batchNo, taskStatus, oldTaskStatus);
    }
    /**
     * desc:更新任务为执行中.<br/>
     * @param lastExecTime lastExecTime
     * @param lastHost lastHost
     * @param tasks tasks
     * @return int
     */
    
    public int updateTaskExec(Date lastExecTime,String lastHost,List<TrTaskDO> tasks){
    if(tasks!=null && tasks.size()>201){throw new RuntimeException("tasks 超出201长度限制");}
        return trTaskDOMapper.updateTaskExec(lastExecTime, lastHost, tasks);
    }
    /**
     * desc:重置状态，更新下次执行时间.<br/>
     * @param nextExecTime nextExecTime
     * @param taskId taskId
     * @param memo memo
     * @return int
     */
    
    public int updateTaskExep(Date nextExecTime,Long taskId,String memo){
        return trTaskDOMapper.updateTaskExep(nextExecTime, taskId, memo);
    }
    /**
     * desc:根据ip重置任务.<br/>
     * @param lastHost lastHost
     * @param packType packType
     * @return int
     */
    
    public int updateHangTasksByIp(String lastHost,String packType){
        return trTaskDOMapper.updateHangTasksByIp(lastHost, packType);
    }
    /**
     * desc:查看转让任务是否都处理完.<br/>
     * @param batchNo batchNo
     * @return Long
     */
    
    public Long cntInitTaskByBatchNo(String batchNo){
        return trTaskDOMapper.cntInitTaskByBatchNo(batchNo);
    }
    /**
     * desc:查询执行次数超次数的转让任务.<br/>
     * @param syncDate syncDate
     * @param execTimes execTimes
     * @param list list
     * @return List<TransferFailTaskStat>
     */
    
    public List<TransferFailTaskStat> queryFailTRTask(Date syncDate,Long execTimes,List<String> list){
    if(list!=null && list.size()>201){throw new RuntimeException("list 超出201长度限制");}
        return trTaskDOMapper.queryFailTRTask(syncDate, execTimes, list);
    }
    /**
     * desc:查询执行次数超次数的转让任务数量.<br/>
     * @param syncDate syncDate
     * @param execTimes execTimes
     * @param packType packType
     * @return Long
     */
    
    public Long countFailTrTask(Date syncDate,Long execTimes,String packType){
        return trTaskDOMapper.countFailTrTask(syncDate, execTimes, packType);
    }
    /**
     * desc:清理历史数据.<br/>
     * @param tfrDate tfrDate
     * @param tfrOut tfrOut
     * @param packType packType
     * @return int
     */
    
    public int deleteHisTask(Date tfrDate,String tfrOut,String packType){
        return trTaskDOMapper.deleteHisTask(tfrDate, tfrOut, packType);
    }
    /**
     * desc:清理历史数据.<br/>
     * @param tfrOut tfrOut
     * @param packType packType
     * @return int
     */
    
    public int deleteUnSuccTask(String tfrOut,String packType){
        return trTaskDOMapper.deleteUnSuccTask(tfrOut, packType);
    }
    /**
     * desc:获取任务数量.<br/>
     * @param tfrOut tfrOut
     * @param packType packType
     * @param statusList statusList
     * @return Long
     */
    
    public Long cntTrTaskByTfrOut(String tfrOut,String packType,List<String> statusList){
    if(statusList!=null && statusList.size()>201){throw new RuntimeException("statusList 超出201长度限制");}
        return trTaskDOMapper.cntTrTaskByTfrOut(tfrOut, packType, statusList);
    }
}
