package com.abssqr.plat.common.dal.mysql.auto.mapper;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanDO;
import java.util.List;
import java.util.Map;
import com.abssqr.plat.common.dal.mysql.auto.paging.PlanListPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanListResultMap;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstServiceReport;
import java.util.Date;
import org.apache.ibatis.annotations.Param;

/**
 * 由于需要对分页支持,请直接使用对应的DAO类
 * The Table abs_plan.
 * 计划信息
 */
public interface PlanDOMapper{

    /**
     * desc:插入表:abs_plan.<br/>
     * @param entity entity
     * @return int
     */
    int insert(PlanDO entity);
    /**
     * desc:根据唯一约束PlanNo更新表:abs_plan.<br/>
     * @param entity entity
     * @return int
     */
    int updateByPlanNo(PlanDO entity);
    /**
     * desc:根据唯一约束PlanNo更新表:abs_plan.<br/>
     * @param planNo planNo
     * @param status status
     * @param gmtModifier gmtModifier
     * @param approveStatus approveStatus
     * @return int
     */
    int updateStatusByPlanNo(@Param("planNo")String planNo,@Param("status")String status,@Param("gmtModifier")String gmtModifier,@Param("approveStatus")String approveStatus);
    /**
     * desc:根据唯一约束PlanNo修改状态:abs_plan.<br/>
     * @param planNo planNo
     * @param passed passed
     * @return int
     */
    int updatePassed(@Param("planNo")String planNo,@Param("passed")Boolean passed);
    /**
     * desc:根据唯一约束PlanNo删除数据:abs_plan.<br/>
     * @param planNo planNo
     * @return int
     */
    int deleteByPlanNo(@Param("planNo")String planNo);
    /**
     * desc:根据唯一约束PlanNo获取数据:abs_plan.<br/>
     * @param planNo planNo
     * @return PlanDO
     */
    PlanDO getByPlanNo(@Param("planNo")String planNo);
    /**
     * desc:根据唯一约束PlanNo获取数据:abs_plan.<br/>
     * @param planNo planNo
     * @return PlanDO
     */
    PlanDO lockByPlanNo(@Param("planNo")String planNo);
    /**
     * desc:根据唯一约束PlanNo获取数据:abs_plan.<br/>
     * @param list list
     * @return List<PlanDO>
     */
    List<PlanDO> queryByPlanNo(List<String> list);
    /**
     * desc:新增判断计划是否存在.<br/>
     * @param planNo planNo
     * @param planName planName
     * @return Integer
     */
    Integer planExist(@Param("planNo")String planNo,@Param("planName")String planName);
    /**
     * desc:编辑判断计划是否存在.<br/>
     * @param planNo planNo
     * @param planName planName
     * @return Integer
     */
    Integer getSamePlan(@Param("planNo")String planNo,@Param("planName")String planName);
    /**
     * desc:分页查询计划.<br/>
     * @param planList planList
     * @return int
     */
    int getPlanListCount(PlanListPage planList);
    /**
     * desc:分页查询计划.<br/>
     * @param planList planList
     * @return List<PlanListResultMap>
     */
    List<PlanListResultMap> getPlanListResult(PlanListPage planList);
    /**
     * desc:根据状态数组获取计划列表.<br/>
     * @param planType planType
     * @param planNos planNos
     * @param statusList statusList
     * @return List<PlanDO>
     */
    List<PlanDO> getPlansByStatusList(@Param("planType")String planType,@Param("planNos")List<String> planNos,@Param("statusList")List<String> statusList);
    /**
     * desc:根据状态数组获取计划列表.<br/>
     * @param status status
     * @param planType planType
     * @return Long
     */
    Long getAllPlanAmt(@Param("status")String status,@Param("planType")String planType);
    /**
     * desc:查询资产服务报告.<br/>
     * @param planNo planNo
     * @return AstServiceReport
     */
    AstServiceReport getAstServiceReport(@Param("planNo")String planNo);
    /**
     * desc:根据服务机构获取存续期受让计划.<br/>
     * @param managerOrgCode managerOrgCode
     * @return List<PlanDO>
     */
    List<PlanDO> getSpvByManagerOrgCode(@Param("managerOrgCode")String managerOrgCode);
    /**
     * desc:查询每日任务失败的数量.<br/>
     * @param curDate curDate
     * @return Long
     */
    Long getFailDailyCount(@Param("curDate")Date curDate);
    /**
     * desc:更新定价规则.<br/>
     * @param planNo planNo
     * @param priceType priceType
     * @return int
     */
    int updatePriceType(@Param("planNo")String planNo,@Param("priceType")String priceType);
    /**
     * desc:根据类型获取所有的计划.<br/>
     * @param planType planType
     * @return List<PlanDO>
     */
    List<PlanDO> getAllPlanByType(@Param("planType")String planType);
}
