/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.facade.enums;

import cn.hutool.core.collection.CollectionUtil;
import com.general.system.common.model.IEnum;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 统计业务类型
 * @author hanfei
 * @version $Id: AnalyseServiceEnum.java, v 0.1 2019-09-19 12:06 AM hanfei Exp $ 
 */
public enum AnalyseServiceEnum implements IEnum {
    
    /**
     * 资产情况
     */
    // 放款池、承接池报表的逾期剩余本金和 (WK012) / 放款池、承接池资产剩余本金和 (WK017)
    DAY_OVD("ana101", "整体逾期率", AnalyseTypeEnum.AMT_RATE),
    // 放款池、承接池早偿金额和 (WK042+WK043) / 放款池、承接吃资产剩余本金和 (WK017)
    DAY_PRE("ana102", "早偿率", AnalyseTypeEnum.AMT_RATE),
    // "WK111", "新增贷款发生额"
    GRANT("ana103", "新增放贷金额", AnalyseTypeEnum.AMT),
    // "WK028", "当日交易贷款发生额"
    TRANS("ana104", "转让资产金额", AnalyseTypeEnum.AMT),
    // "WK017", "总贷款本金余额"
    TOTAL_SCALE("ana105", "总资产规模", AnalyseTypeEnum.AMT),
    
    /**
     * 资金情况
     */
    // "WK001", "闲置资金余额"
    BAL_FUND("ana201", "资金余额", AnalyseTypeEnum.AMT),
    //
    OUT_FUND("ana202", "资金流出", AnalyseTypeEnum.AMT),
    //
    IN_FUND("ana203", "资金流入", AnalyseTypeEnum.AMT),
    //
    DAY_IDLE("ana204", "每日闲置率", AnalyseTypeEnum.AMT_RATE),
    
    /*
     流出项
     "WK111", "新增贷款发生额"
     "WK005", "兑付流出资金发生额"
     "WK108", "当日金融服务费"
     "WK006", "其他流出资金发生额"
     流入项
     "WK112", "转让贷款资金发生额"
     "WK039", "当日已还本金发生额"+ "WK040", "当日已还利息发生额"+ "WK041", "当日已还罚息发生额" --- 项目级别
     "WK100", "应计支付应付利息"
     "WK101", "放款冲正金额"
     "WK007", "其他流入资金发生额"
     */
    DAY_FLOW("ana205", "资金情况流动", AnalyseTypeEnum.AMT),
    
    
    
    //
    ;
    
    private static final Map<String, AnalyseServiceEnum> codeMap = new HashMap<>();
    
    static {
        for (AnalyseServiceEnum item : AnalyseServiceEnum.values()) {
            codeMap.put(item.getCode(), item);
        }
    }
    
    private String code;
    private String desc;
    /**
     * 统计类型单位
     */
    private AnalyseTypeEnum type;
    
    AnalyseServiceEnum(String code, String desc, AnalyseTypeEnum type) {
        this.code = code;
        this.desc = desc;
        this.type = type;
    }
    
    public static AnalyseServiceEnum getByCode(String code) {
        return codeMap.get(code);
    }
    
    public static List<AnalyseServiceEnum> getByCodes(List<String> codes) {
        if (CollectionUtil.isNotEmpty(codes)){
            List<AnalyseServiceEnum> serviceEnums = new ArrayList<>(codes.size());
            for (String code : codes){
                AnalyseServiceEnum serviceEnum = AnalyseServiceEnum.getByCode(code);
                if (serviceEnum != null){
                    serviceEnums.add(serviceEnum);
                }
            }
            return serviceEnums;
        }
        return null;
    }
    @Override
    public String getCode() {
        return code;
    }
    
    @Override
    public String getDesc() {
        return desc;
    }
    
    public AnalyseTypeEnum getType() {
        return type;
    }
}
