/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.enums;

import com.general.system.common.model.IEnum;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 科目类型
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.enums: FundTypeEnum.java, v 0.1 2018-01-03 21:57 hongwei.wang Exp $
 */
public enum TitleTypeEnum implements IEnum {
    PRIOR_INVESTED("TT001", "优先级-投入资本", AllocTypeEmun.PRIN, 2),
    SECND_INVESTED("TT002", "劣后级-投入资本", AllocTypeEmun.PRIN, 3),
    INVESTED("TT003", "投入资本", null, 888),
    FEE("TT004", "费用", AllocTypeEmun.FEE, 0),
    INVEST_PROFIT("TT005", "投资人收益", AllocTypeEmun.INVEST, 1),
    COST("TT006", "支出", null, 999),
    ;

    private static final Map<String, TitleTypeEnum> codeMap = new HashMap<String, TitleTypeEnum>();

    static {
        for (TitleTypeEnum item : TitleTypeEnum.values()) {
            codeMap.put(item.getCode(), item);
        }
    }

    private String code;
    private String desc;
    private int    order;
    private AllocTypeEmun allocType;

    TitleTypeEnum(String code, String desc, AllocTypeEmun allocType, int order) {
        this.code = code;
        this.desc = desc;
        this.order = order;
        this.allocType=allocType;
    }

    public static TitleTypeEnum getByCode(String code) {
        return codeMap.get(code);
    }

    @Override
    public String getCode() {
        return code;
    }

    @Override
    public String getDesc() {
        return desc;
    }

    public int getOrder() {
        return order;
    }

    public AllocTypeEmun getAllocType() {
        return allocType;
    }

    public boolean isInvest() {
        return this == PRIOR_INVESTED || this == SECND_INVESTED || this == INVESTED;
    }
    public static List<TitleTypeEnum> convert(AllocTypeEmun allocType){
       return codeMap.values().stream().filter(titleTypeEnum -> titleTypeEnum.getAllocType()==allocType).collect(
           Collectors.toList());
    }
}