/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.model.plan;

import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.FeeTimeTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanAllocStatusEnum;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 分配计划
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.domain.plan: PlanAlloc.java, v 0.1 2019-06-03 16:05 bangis.wangdf Exp $
 */
public class PlanAlloc extends BaseEntity implements Serializable {
    private static final long serialVersionUID = 2724871818218559013L;
    /**
     * 计划类型
     */
    private PlanTypeEnum planType;
    /**
     * 计划编号
     */
    private String planNo;
    /**
     * allocNo 分配编号.
     */
    private String allocNo;
    /**
     * clearDate 结算日.
     */
    private Date clearDate;
    /**
     * payDate 兑付日.
     */
    private Date payDate;
    /**
     * 分配状态
     */
    private PlanAllocStatusEnum status;
    /**
     * allocAmt 本期分配总额.
     */
    private Money allocAmt = new Money(0);
    /**
     * planApproveStatus 审批状态
     * OUTOFDATE
     * PASSED
     * FAILED
     * PENDING
     * 已过时
     * 已通过
     * 已失败
     * 审批中.
     */
    private ApproveStatusEnum approveStatus;
    /**
     * 分配事项
     */
    private List<PlanAllocItem> planAllocItemList = Lists.newArrayList();
    /**
     * 分配日期类型
     */
    private FeeTimeTypeEnum timeType;

    /**
     * 分配金额 获取分配事项合计金额
     * 注：若 cfrmAmt 不为0，用cfrmAmt，否则用 trialAmt
     *
     * @return 累计需要冻结金额
     */
    public Money calcTotalTrialAmt() {
        Money total = new Money();
        for (PlanAllocItem item : planAllocItemList) {
            if(item.getPayTrial()) {
                // 累加 试算金额 或 确认金额
                total.addTo(approveStatus == ApproveStatusEnum.PASSED
                    || item.getConfirmAmt().isGreaterThanZero() ? item.getConfirmAmt() : item.getTrialAmt());
            }
            // 累加 试算增值税 或 确认增值税
            total.addTo(approveStatus == ApproveStatusEnum.PASSED
                    || item.getConfirmTaxAmt().isGreaterThanZero() ? item.getConfirmTaxAmt() : item.getTrialTaxAmt());
        }
        return total;
    }

    /**
     * 分配确认金额
     *
     * @param items
     * @return
     */
    public static Money calcPayAmt(List<PlanAllocItem> items) {
        Money total = new Money();
        for (PlanAllocItem item : items) {
            Money amt = item.getConfirmAmt();
            item.getSettleAmt().addTo(amt);
            // 累加 确认金额
            total.addTo(amt);
        }
        return total;
    }

    /**
     * 分配确认费用
     *
     * @param items
     * @return
     */
    public static Money calcPayVTaxAmt(List<PlanAllocItem> items) {
        Money total = new Money();
        for (PlanAllocItem item : items) {
            Money tax = item.getConfirmTaxAmt();
            item.getSettleAmt().addTo(tax);
            // 累加 确认增值税
            total.addTo(tax);
        }
        return total;
    }

    /**
     * 获取确认分配金额
     *
     * @return
     */
    public Money getTotalConfirmAllocAmt() {
        Money total = new Money();
        for (PlanAllocItem item : planAllocItemList) {
            // 累加 确认金额
            if (item.getPayTrial()) {
                total.addTo(item.getConfirmAmt());
            }
            // 累加 确认增值税
            total.addTo(item.getConfirmTaxAmt());
        }
        return total;
    }

    public void sortOrders() {
        if (CollectionUtils.isNotEmpty(this.getPlanAllocItemList())) {
            for (int i = 0; i < this.getPlanAllocItemList().size(); i++) {
                this.getPlanAllocItemList().get(i).setOrder(i + 1);
            }
        }
    }

    public void addPlanAllocItem(PlanAllocItem planAllocItem) {
        planAllocItemList.add(planAllocItem);
    }

    public String getAllocNo() {
        return allocNo;
    }

    public void setAllocNo(String allocNo) {
        this.allocNo = allocNo;
    }

    public Date getClearDate() {
        return clearDate;
    }

    public void setClearDate(Date clearDate) {
        this.clearDate = clearDate;
    }

    public Date getPayDate() {
        return payDate;
    }

    public void setPayDate(Date payDate) {
        this.payDate = payDate;
    }

    public Money getAllocAmt() {
        return allocAmt;
    }

    public void setAllocAmt(Money allocAmt) {
        this.allocAmt = allocAmt;
    }

    public List<PlanAllocItem> getPlanAllocItemList() {
        return planAllocItemList;
    }

    public void setPlanAllocItemList(List<PlanAllocItem> planAllocItemList) {
        this.planAllocItemList = planAllocItemList;
    }

    public String getPlanNo() {
        return planNo;
    }

    public void setPlanNo(String planNo) {
        this.planNo = planNo;
    }

    public PlanAllocStatusEnum getStatus() {
        return status;
    }

    public void setStatus(PlanAllocStatusEnum status) {
        this.status = status;
    }

    public ApproveStatusEnum getApproveStatus() {
        return approveStatus;
    }

    public void setApproveStatus(ApproveStatusEnum approveStatus) {
        this.approveStatus = approveStatus;
    }

    public FeeTimeTypeEnum getTimeType() {
        return timeType;
    }

    public void setTimeType(FeeTimeTypeEnum timeType) {
        this.timeType = timeType;
    }

    public PlanTypeEnum getPlanType() {
        return planType;
    }

    public void setPlanType(PlanTypeEnum planType) {
        this.planType = planType;
    }
}
