/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */

package com.abssqr.plat.common.facade.param.ast;

import com.abssqr.plat.common.facade.base.BaseOprParam;
import com.abssqr.plat.common.facade.enums.AstTypeEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.param.ast.sub.AstScreenRuleParam;
import com.abssqr.plat.common.facade.validation.VldCollection;
import com.abssqr.plat.common.facade.validation.VldEnums;
import com.abssqr.plat.common.facade.validation.VldMoney;
import com.abssqr.plat.common.facade.validation.VldString;
import com.general.system.common.model.Money;

import java.util.ArrayList;
import java.util.List;

/**
 * @author xiachenxiang
 * @version com.abssqr.plat.common.facade.param.ast: AstPaclOprParam.java, v 0.1 2020-04-18 3:31 PM xiachenxiang Exp $
 */
public class AstPackOprParam extends BaseOprParam {

    /**
     * 资产包编号
     */
    @VldString(message = "资产包编号不能为空")
    private String packNo;

    /**
     * 资产包名
     */
    @VldString(message = "资产包名称不能为空")
    private String packName;
    /**
     * 规模
     */
    @VldString(message = "规模不能为空")
    private String avalAmt = "0";

    /**
     * 筛选规则
     */
    @VldCollection(nullable = true)
    private List<AstScreenRuleParam> screenRuleList = new ArrayList<>();

    /**
     * 资产池编号
     */
    @VldString(message = "资产池编号不能为空")
    private String astPoolNo;

    /**
     * 资产类型
     */
    @VldEnums(enumClass = AstTypeEnum.class, message = "不支持的资产类型")
    private String astType;

    public String getPackNo() {
        return packNo;
    }

    public void setPackNo(String packNo) {
        this.packNo = packNo;
    }

    public String getPackName() {
        return packName;
    }

    public void setPackName(String packName) {
        this.packName = packName;
    }

    public String getAvalAmt() {
        return avalAmt;
    }

    public void setAvalAmt(String avalAmt) {
        this.avalAmt = avalAmt;
    }

    public Money getAvalAmtMoney(){
        return new Money(avalAmt);
    }
    public List<AstScreenRuleParam> getScreenRuleList() {
        return screenRuleList;
    }

    public void setScreenRuleList(List<AstScreenRuleParam> screenRuleList) {
        this.screenRuleList = screenRuleList;
    }

    public String getAstPoolNo() {
        return astPoolNo;
    }

    public void setAstPoolNo(String astPoolNo) {
        this.astPoolNo = astPoolNo;
    }

    public String getAstType() {
        return astType;
    }

    public void setAstType(String astType) {
        this.astType = astType;
    }

    @Override
    public EventCodeEnum getEventCode() {
        return EventCodeEnum.ADMIN_EDIT;
    }
}
