/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.facade.utils;

import com.abssqr.plat.common.facade.model.report.DailyReportDateField;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.manRpt.AmtPropCnt;
import com.abssqr.plat.common.facade.model.report.manRpt.AmtPropCntGroup;
import com.abssqr.plat.common.facade.model.report.manRpt.CommonPropStat;
import com.abssqr.plat.common.facade.model.report.manRpt.CommonPropStatGroup;
import com.general.system.common.util.DateTimeUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 导出模版数据格式转换
 * @author zhenxuan.luo
 * @version com.ruiyi.rdassist.biz.shared.utils: TmplUtils.java, v 0.1 2019-02-28 17:43 zhenxuan.luo Exp $ 
 */
@Component
public class TmplUtils {
    
    public final static String BLANK = "--";
    
    public int incrId(ListID id) {
        return id.incr();
    }
    
    public String rptDt(DailyReportDateField dateField) {
        if (null == dateField) {
            return BLANK;
        } else {
            return dateField.desc();
        }
    }
    
    public String shortDate(Date date) {
        if (null == date) {
            return BLANK;
        } else {
            return DateTimeUtil.formatY_M_D(date);
        }
    }
    
    public String formatAmt(BigDecimal amt) {
        if (null == amt) {
            return BLANK;
        } else {
            return BigDecimalFormatUtil.formatAmt(amt);
        }
    }
    
    public String formatRate(BigDecimal amt) {
        if (null == amt) {
            return BLANK;
        } else {
            return BigDecimalFormatUtil.formatRate(amt) + "%";
        }
    }
    
    public String formatCnt(BigDecimal amt) {
        if (null == amt) {
            return BLANK;
        } else {
            return BigDecimalFormatUtil.formatCnt(amt);
        }
    }
    
    public String getAmt(PlanReport report, String key) {
        
        PlanReportItem item = report.getReportItemMap().get(key);
        if (null == item) {
            return BLANK;
        } else {
            return BigDecimalFormatUtil.formatAmt(item.getAmt());
        }
    }
    
    public String getAstRate(PlanReport report, String key) {
        
        PlanReportItem item = report.getReportItemMap().get(key);
        if (null == item) {
            return BLANK;
        } else {
            return BigDecimalFormatUtil.formatRate(item.getAmt()) + "%";
        }
    }
    
    public String getCnt(PlanReport report, String key) {
        
        PlanReportItem item = report.getReportItemMap().get(key);
        if (null == item) {
            return BLANK;
        } else {
            return String.valueOf(item.getAstCnt());
        }
    }
    
    public String getPropStatMin(CommonPropStatGroup commonPropStatGroup, String key) {
        CommonPropStat propStat = commonPropStatGroup.getCommonPropStat().get(key);
        if (null == propStat) {
            return BLANK;
        } else {
            return BigDecimalFormatUtil.formatAmt(propStat.getMin());
        }
    }
    
    public String getPropStatMax(CommonPropStatGroup commonPropStatGroup, String key) {
        CommonPropStat propStat = commonPropStatGroup.getCommonPropStat().get(key);
        if (null == propStat) {
            return BLANK;
        } else {
            return BigDecimalFormatUtil.formatAmt(propStat.getMax());
        }
    }
    
    public String getPropStatAvg(CommonPropStatGroup commonPropStatGroup, String key) {
        CommonPropStat propStat = commonPropStatGroup.getCommonPropStat().get(key);
        if (null == propStat) {
            return BLANK;
        } else {
            return BigDecimalFormatUtil.formatAmt(propStat.getAvg());
        }
    }
    
    public String getPropStatWAvg(CommonPropStatGroup commonPropStatGroup, String key) {
        CommonPropStat propStat = commonPropStatGroup.getCommonPropStat().get(key);
        if (null == propStat) {
            return BLANK;
        } else {
            return BigDecimalFormatUtil.formatAmt(propStat.getwAvg());
        }
    }
    
    public String getPropStatRateMin(CommonPropStatGroup commonPropStatGroup, String key) {
        CommonPropStat propStat = commonPropStatGroup.getCommonPropStat().get(key);
        if (null == propStat) {
            return BLANK;
        } else {
            return formatRate(propStat.getMin());
        }
    }
    
    public String getPropStatRateMax(CommonPropStatGroup commonPropStatGroup, String key) {
        CommonPropStat propStat = commonPropStatGroup.getCommonPropStat().get(key);
        if (null == propStat) {
            return BLANK;
        } else {
            return formatRate(propStat.getMax());
        }
    }
    
    public String getPropStatRateAvg(CommonPropStatGroup commonPropStatGroup, String key) {
        CommonPropStat propStat = commonPropStatGroup.getCommonPropStat().get(key);
        if (null == propStat) {
            return BLANK;
        } else {
            return formatRate(propStat.getAvg());
        }
    }
    
    public String getPropStatRateWAvg(CommonPropStatGroup commonPropStatGroup, String key) {
        CommonPropStat propStat = commonPropStatGroup.getCommonPropStat().get(key);
        if (null == propStat) {
            return BLANK;
        } else {
            return formatRate(propStat.getwAvg());
        }
    }
    
    public String getPropCntAmt(AmtPropCntGroup amtPropCntGroup, String key, int index) {
        List<AmtPropCnt> amtPropCnts = amtPropCntGroup.getPropCntMap().get(key);
        if (CollectionUtils.isEmpty(amtPropCnts)) {
            return BLANK;
        }
        
        if (index < 0 || index >= amtPropCnts.size()) {
            return BLANK;
        }
        
        return BigDecimalFormatUtil.formatAmt(amtPropCnts.get(index).getPrin());
    }
    
    public String getPropCntCnt(AmtPropCntGroup amtPropCntGroup, String key, int index) {
        List<AmtPropCnt> amtPropCnts = amtPropCntGroup.getPropCntMap().get(key);
        if (CollectionUtils.isEmpty(amtPropCnts)) {
            return BLANK;
        }
    
        if (index < 0 || index >= amtPropCnts.size()) {
            return BLANK;
        }
        
        return BigDecimalFormatUtil.formatCnt(amtPropCnts.get(index).getCnt());
    }
    
    public String getPropCntName(AmtPropCntGroup amtPropCntGroup, String key, int index) {
        List<AmtPropCnt> amtPropCnts = amtPropCntGroup.getPropCntMap().get(key);
        if (CollectionUtils.isEmpty(amtPropCnts)) {
            return BLANK;
        }
    
        if (index < 0 || index >= amtPropCnts.size()) {
            return BLANK;
        }
        
        return amtPropCnts.get(index).getPropVal();
    }
}