/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstSyncTaskCtrDO;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.domain.task.ctr.ext.AstSyncTaskContext;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.alibaba.fastjson.JSON;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.apache.commons.lang3.StringUtils;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.convertor: AstSyncTaskCtrConvert.java, v 0.1 2019-06-14 12:42 bangis.wangdf
 * Exp $
 */
public class AstSyncTaskCtrConvert {
    
    public static AstSyncTaskCtr convert2Domain(AstSyncTaskCtrDO syncTaskCtrDO) {
        if (syncTaskCtrDO == null){
            return null;
        }
        AstSyncTaskCtr entity = new AstSyncTaskCtr();
        entity.setOrgCode(syncTaskCtrDO.getOrgCode());
        entity.setEff(syncTaskCtrDO.getEff());
        if (StringUtils.isNotBlank(syncTaskCtrDO.getExtData())) {
            entity.setExtData(JSON.parseObject(syncTaskCtrDO.getExtData(), AstSyncTaskContext.class));
        }
        entity.setTaskCode(syncTaskCtrDO.getTaskCode());
        return entity;
    }

    public static AstSyncTaskCtrDO convert2Do(AstSyncTaskCtr astSyncTaskCtr) {
        if (astSyncTaskCtr == null){
            return null;
        }
        AstSyncTaskCtrDO entity = new AstSyncTaskCtrDO();
        entity.setOrgCode(astSyncTaskCtr.getOrgCode());
        entity.setEff(astSyncTaskCtr.getEff());
        entity.setExtData(JSON.toJSONString(astSyncTaskCtr.getExtData()));
        if(StringUtils.isBlank(astSyncTaskCtr.getTaskCode())) {
            entity.setTaskCode(SequenceUtil.genId(IdTypeEnum.SEQ));
        }else{
            entity.setTaskCode(astSyncTaskCtr.getTaskCode());
        }
        return entity;
    }

    public static AstSyncTaskCtr initSyncTaskCtr(String orgCode, String entityCode) {
        OrgCodeEnum orgCodeEnum = OrgCodeEnum.getByCode(orgCode);
        VarChecker.checkNotNull(orgCodeEnum, "不匹配的机构编码={0}", orgCode);
        AstSyncTaskContext context = new AstSyncTaskContext();
        context.setCtrFlg(orgCodeEnum.getCtrFlg());
        context.setSyncAll(false);
        // 设置对应放款池计划编码
        context.setSpvPlanNos(Sets.newHashSet(Lists.newArrayList(entityCode)));
        
        AstSyncTaskCtr entity = new AstSyncTaskCtr();
        entity.setOrgCode(orgCodeEnum.getCode());
        entity.setEff(false);
        entity.setExtData(context);
        entity.setTaskCode(SequenceUtil.genId(IdTypeEnum.SEQ));
        return entity;
    }
}
