/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.LoanInstalDO;
import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.OwnStatusEnum;
import com.abssqr.plat.common.model.domain.ast.AstOrg;
import com.abssqr.plat.common.model.domain.ast.AstPeriod;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.utils.TextUtil;
import com.general.system.common.model.Money;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;

import java.util.List;

/**
 * 账单分期明细与测算模块的还款计划模型互相转换
 *
 * @author yaphet
 * @version com.abssqr.plat.common.model.convertor: LoanInstalConvert.java, v 0.1 2018-06-20 上午10:53 yaphet Exp $
 */
public class LoanInstalConvert {
    
    public static List<LoanInstalDomain> convert2Domains(List<LoanInstalDO> vars, Loan loan) {
        List<LoanInstalDomain> loanInstals = Lists.newArrayList();
        
        if (CollectionUtils.isEmpty(vars)) {
            return loanInstals;
        }
        
        for (LoanInstalDO loanInstalDO : vars) {
            loanInstals.add(convert2Domain(loanInstalDO, loan));
        }
        
        return loanInstals;
    }
    
    public static LoanInstalDomain convert2Domain(LoanInstalDO var, Loan loan) {
        
        if (null == var) {
            return null;
        }
    
        LoanInstalDomain instance = new LoanInstalDomain(false);
        instance.setRefLoan(loan);
        instance.setUserId(var.getUserId());
        instance.setAstNo(var.getAstNo());
        instance.setOutAstNo(var.getOutAstNo());
        instance.setContNo(var.getContractNo());
        instance.setProdNo(var.getProdNo());
        instance.setOutProdNo(var.getOutProdNo());
        instance.setOwnPack(var.getOwnPack());
        instance.setTermNo(var.getTermNo());
        instance.setDueIntrAmt(var.getDueIntrAmt());
        instance.setDueFeeAmt(var.getDueFeeAmt());
        instance.setBal(createBal(var));
        instance.setPaidAmt(createPaidAmt(var));
        instance.setPaidFundAmt(createPaidFundAmt(var));
        instance.setRmtAmt(createRmtAmt(var));
        instance.setStatus(LoanStatusEnum.getByCode(var.getStatus()));
        instance.setOvdDays(var.getOvdDays());
        instance.setLastUpdateDate(var.getLastUpdateDate());
        instance.setExtInf(TextUtil.convertTxt2Map(var.getExtInf()));
        instance.getAstPeriod().setStartDate(var.getStartDate());
        instance.getAstPeriod().setEndDate(var.getEndDate());
        instance.getAstPeriod().setClearDate(var.getClearDate());
        instance.getAstPeriod().setInDate(var.getInDate());
        instance.getAstPeriod().setGraceEndDate(var.getGraceEndDate());
        instance.getAstOrg().setSponsorOrgCode(var.getSponserOrgCode());
        instance.getAstOrg().setManagerOrgCode(var.getManagerOrgCode());
        instance.getAstOrg().getAstOwnership().setLenderOrgCode(var.getLenderOrgCode());
        instance.getAstOrg().getAstOwnership().setLastOwnOrg(var.getLastOwnOrg());
        instance.getAstOrg().getAstOwnership().setOwnOrg(var.getOwnOrg());
        instance.getAstOrg().getAstOwnership().setOwnStatus(OwnStatusEnum.getByCode(var.getOwnStatus()));
        
        instance.setHangIntrBal(var.getHangIntrBal());
        instance.setHangIntrBak(var.getHangIntrBal().clone());
        instance.setPaidHangIntrAmt(var.getPaidHangIntrAmt()==null?new Money():var.getPaidHangIntrAmt());
        instance.setCost(var.getCost());
        
        createAstPeriod(var, instance.getAstPeriod());
        createAstOrg(var, instance.getAstOrg());
        
        return instance;
        
    }
    
    public static LoanInstalDO convert2Do(LoanInstal var) {
        
        if (null == var) {
            return null;
        }
        
        LoanInstalDO instance = new LoanInstalDO();
        instance.setUserId(var.getUserId());
        instance.setAstNo(var.getAstNo());
        instance.setOutAstNo(var.getOutAstNo());
        instance.setProdNo(var.getProdNo());
        instance.setOutProdNo(var.getOutProdNo());
        instance.setContractNo(var.getContNo());
        
        instance.setSponserOrgCode(var.getAstOrg().getSponsorOrgCode());
        instance.setManagerOrgCode(var.getAstOrg().getManagerOrgCode());
        instance.setLenderOrgCode(var.getAstOrg().getAstOwnership().getLenderOrgCode());
        instance.setOwnOrg(var.getAstOrg().getAstOwnership().getOwnOrg());
        instance.setLastOwnOrg(var.getAstOrg().getAstOwnership().getLastOwnOrg());
        instance.setOwnStatus(var.getAstOrg().getAstOwnership().getOwnStatus().getCode());
        instance.setOwnPack(var.getOwnPack());
        
        instance.setTermNo(var.getTermNo());
        
        instance.setStartDate(var.getAstPeriod().getStartDate());
        instance.setEndDate(var.getAstPeriod().getEndDate());
        instance.setClearDate(var.getAstPeriod().getClearDate());
        instance.setGraceEndDate(var.getAstPeriod().getGraceEndDate());
        instance.setInDate(var.getAstPeriod().getInDate());
        
        instance.setDueIntrAmt(var.getDueIntrAmt());
        instance.setDueFeeAmt(var.getDueFeeAmt());
        
        instance.setOvdDays(var.getOvdDays());
        instance.setStatus(var.getStatus().getCode());
        
        instance.setLastUpdateDate(var.getLastUpdateDate());
        instance.setExtInf(TextUtil.convertMap2Txt(var.getExtInf()));
        
        instance.setPrinBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN));
        instance.setPrinPenalBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setIntrBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setIntrPenalBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setFeeBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setFeePenalBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        instance.setOtherFeeBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE));

        instance.setPaidPrinAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.PRIN));
        instance.setPaidPrinPenalAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setPaidIntrAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setPaidIntrPenalAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setPaidFeeAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setPaidFeePenalAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        instance.setPaidOtherFeeAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE));

        instance.setRmtPrinPenalAmt(var.getRmtAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setRmtIntrAmt(var.getRmtAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setRmtIntrPenalAmt(var.getRmtAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setRmtFeeAmt(var.getRmtAmt().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setRmtFeePenalAmt(var.getRmtAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
    
        instance.setPaidPrinFundAmt(var.getPaidFundAmt().getByFieldType(DebtMoneyFieldEnum.PRIN));
        instance.setPaidPrinPenalFundAmt(var.getPaidFundAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setPaidIntrFundAmt(var.getPaidFundAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setPaidIntrPenalFundAmt(var.getPaidFundAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setPaidFeeFundAmt(var.getPaidFundAmt().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setPaidFeePenalFundAmt(var.getPaidFundAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        instance.setPaidOtherFeeFundAmt(var.getPaidFundAmt().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE));
        
        instance.setHangIntrBal(var.getHangIntrBal());
        instance.setCost(var.getCost());
        instance.setPaidHangIntrAmt(var.getPaidHangIntrAmt());
        return instance;
        
    }
    
    private static void createAstPeriod(LoanInstalDO loanInstalDO, AstPeriod astPeriod) {
        astPeriod.setClearDate(loanInstalDO.getClearDate());
        astPeriod.setEndDate(loanInstalDO.getEndDate());
        astPeriod.setStartDate(loanInstalDO.getStartDate());
    }
    
    private static void createAstOrg(LoanInstalDO loanInstalDO, AstOrg astOrg) {
        astOrg.setSponsorOrgCode(loanInstalDO.getSponserOrgCode());
        astOrg.setManagerOrgCode(loanInstalDO.getManagerOrgCode());
        astOrg.getAstOwnership().setLenderOrgCode(loanInstalDO.getLenderOrgCode());
        astOrg.getAstOwnership().setLastOwnOrg(loanInstalDO.getLastOwnOrg());
        astOrg.getAstOwnership().setOwnOrg(loanInstalDO.getOwnOrg());
    }
    
    private static DebtMoney createBal(LoanInstalDO loanInstalDO) {
        DebtMoney debtMoney = new DebtMoney();
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN, loanInstalDO.getPrinBal());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, loanInstalDO.getPrinPenalBal());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR, loanInstalDO.getIntrBal());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, loanInstalDO.getIntrPenalBal());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE, loanInstalDO.getFeeBal());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, loanInstalDO.getFeePenalBal());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,loanInstalDO.getOtherFeeBal());
        return debtMoney;
    }
    
    private static DebtMoney createPaidAmt(LoanInstalDO loanInstalDO) {
        DebtMoney debtMoney = new DebtMoney();
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN, loanInstalDO.getPaidPrinAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, loanInstalDO.getPaidPrinPenalAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR, loanInstalDO.getPaidIntrAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, loanInstalDO.getPaidIntrPenalAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE, loanInstalDO.getPaidFeeAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, loanInstalDO.getPaidFeePenalAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,loanInstalDO.getPaidOtherFeeAmt());
        return debtMoney;
    }
    
    private static DebtMoney createPaidFundAmt(LoanInstalDO loanInstalDO) {
        DebtMoney debtMoney = new DebtMoney();
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN, loanInstalDO.getPaidPrinFundAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, loanInstalDO.getPaidPrinPenalFundAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR, loanInstalDO.getPaidIntrFundAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, loanInstalDO.getPaidIntrPenalFundAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE, loanInstalDO.getPaidFeeFundAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, loanInstalDO.getPaidFeePenalFundAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,loanInstalDO.getPaidOtherFeeFundAmt());
        return debtMoney;
    }
    
    private static DebtMoney createRmtAmt(LoanInstalDO loanInstalDO) {
        DebtMoney debtMoney = new DebtMoney();
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN, new Money());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, loanInstalDO.getRmtPrinPenalAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR, loanInstalDO.getRmtIntrAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, loanInstalDO.getRmtIntrPenalAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE, loanInstalDO.getRmtFeeAmt());
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, loanInstalDO.getRmtFeePenalAmt());
        
        return debtMoney;
    }
    
}
