/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanDO;
import com.abssqr.plat.common.facade.enums.*;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.Plan;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.VarChecker;

import java.util.Optional;

/**
 * @author bangis.wangdf
 * @version $Id: PlanParamConvert.java, v 0.1 2018-01-20 上午2:56 hanfei Exp $
 */
public class PlanConvert {

    public static <T extends PlanBase> T convert2Domain(PlanDO param) {
        VarChecker.checkNotNull(param, "不存在该计划[{0}]", param);
        T result = null;
        if (PlanTypeEnum.getByCode(param.getPlanType()) == PlanTypeEnum.SPV) {
            result = (T)new OrganizationEntity();
        } else {
            result = (T)new Plan();
        }
        //totalAmt 发行规模
        result.setTotalAmt(Money.createWithCent(param.getTotalAmt()));
        //planNo 计划编号
        result.setPlanNo(param.getPlanNo());
        //status 计划状态
        result.setStatus(PlanStatusEnum.getByCode(param.getStatus()));
        //fundType 募集方式
        result.setFundType(FundTypeEnum.getByCode(param.getFundType()));
        //planName 计划名
        result.setPlanName(param.getPlanName());
        //approveStatus 审批状态
        result.setApproveStatus(ApproveStatusEnum.getEnum(param.getApproveStatus()));
        //managerOrgCode 管理机构
        result.setPlanMgrOrgCode(param.getPlanMgrOrgCode());
        //sponsorOrgCode 发起机构
        result.setSponsorOrgCode(param.getSponsorOrgCode());
        //endDate 计划到期日（信托终止日）计划截止日
        result.setEndDate(param.getEndDate());
        //beginDate 计划设立日（信托设立日）
        result.setBeginDate(DateTimeUtil.getStandardDate(param.getBeginDate()));
        //effectDate 计划起息日
        result.setEffectDate(param.getEffectDate());
        //firstPayDate 首次偿付日 首次兑付日
        result.setFirstPayDate(param.getFirstPayDate());
        //initCalcDate 初始起算日
        result.setInitCalcDate(param.getInitCalcDate());
        //firstClearDate 首次结算日
        result.setFirstClearDate(param.getFirstClearDate());
        //payFrequency 兑付频率
        result.setPayFreq(PlanPayFreqEnum.getByCode(param.getPayFreq()));
        //countCalcType 计息方式
        result.setCalcIntrType(CalcIntrTypeEnum.getByCode(param.getCalcIntrType()));
        //vatRate 增指税率
        result.setVatRate(param.getVatRate());
        result.setLastUpdateDate(param.getLastUpdateDate());
        //GmtCreator  创建者
        result.setGmtCreator(param.getGmtCreator());
        //GmtModifier  修改人  DO转DOMAIN可能为空
        if (param.getGmtModifier() != null) {
            result.setGmtModifier(param.getGmtModifier());
        }
        //Passed  是否已经通过过
        if (param.getPassed() != null) {
            result.setPassed(param.getPassed());
        }
        //计划类型
        result.setPlanType(PlanTypeEnum.getByCode(param.getPlanType()));
        // 定价规则类型
        result.setPriceType(PriceTypeEnum.getByCode(param.getPriceType()));
        // 金融服务费率
        result.setFinSerFeeRate(param.getFinSerFeeRate());
        return result;
    }

    public static PlanDO convert2Do(PlanBase param) {
        PlanDO result = new PlanDO();
        VarChecker.checkNotNull(param);
        //totalAmt 发行规模
        if (param.getTotalAmt() == null) {
            param.setTotalAmt(new Money("0"));
        }
        result.setTotalAmt(param.getTotalAmt().getCent());
        //planNo 计划编号
        result.setPlanNo(param.getPlanNo());
        //status 计划状态
        if (param.getStatus() == null) {
            param.setStatus(PlanStatusEnum.PREPARING);
        }
        result.setStatus(param.getStatus().getCode());
        //fundType 募集方式
        if (param.getFundType() != null) {
            result.setFundType(param.getFundType().getCode());
        }
        //planName 计划名
        result.setPlanName(param.getPlanName());
        //approveStatus 审批状态
        if (param.getApproveStatus() != null) {
            result.setApproveStatus(param.getApproveStatus().getCode());
        }
        //managerOrgCode 管理机构
        result.setPlanMgrOrgCode(param.getPlanMgrOrgCode());
        //sponsorOrgCode 发起机构
        result.setSponsorOrgCode(param.getSponsorOrgCode());
        //endDate 计划到期日（信托终止日）计划截止日
        result.setEndDate(param.getEndDate());
        //beginDate 计划设立日（信托设立日）
        result.setBeginDate(param.getBeginDate());
        //effectDate 计划起息日
        result.setEffectDate(param.getEffectDate());
        //firstPayDate 首次偿付日 首次兑付日
        result.setFirstPayDate(param.getFirstPayDate());
        //initCalcDate 初始起算日
        result.setInitCalcDate(param.getInitCalcDate());
        //firstClearDate 首次结算日
        result.setFirstClearDate(param.getFirstClearDate());
        //payFrequency 兑付频率
        result.setPayFreq(Optional.ofNullable(param.getPayFreq()).map(PlanPayFreqEnum::getCode).orElse(null));
        //countCalcType 计息方式
        result.setCalcIntrType(Optional.ofNullable(param.getCalcIntrType()).map(CalcIntrTypeEnum::getCode).orElse(null));
        //vatRate 增指税率
        result.setVatRate(param.getVatRate());
        result.setLastUpdateDate(param.getLastUpdateDate());
        //GmtCreator  创建人
        if (param.getGmtCreator() != null) {
            result.setGmtCreator(param.getGmtCreator());
        }
        //GmtModifier  修改人
        if (param.getGmtModifier() != null) {
            result.setGmtModifier(param.getGmtModifier());
        }
        //Passed  是否已经通过过
        if (param.isPassed() != null) {
            result.setPassed(param.isPassed());
        }
        //计划类型
        result.setPlanType(Optional.ofNullable(param.getPlanType()).map(PlanTypeEnum::getCode).orElse(null));
        // 定价规则类型
        result.setPriceType(Optional.ofNullable(param.getPriceType()).map(PriceTypeEnum::getCode).orElse(null));
        // 金融服务费率
        result.setFinSerFeeRate(param.getFinSerFeeRate());
        return result;
    }
}
