/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.convertor;

import cn.hutool.core.util.NumberUtil;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanReportDO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanReportItemDO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanReportItemDefaultDO;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.model.report.DailyReportDateField;
import com.abssqr.plat.common.facade.model.report.MonthReportDateField;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportDateField;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 报表模型转换
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.convertor: PlanReportConvert.java, v 0.1 2019-06-28 20:10 zhenxuan.luo Exp $ 
 */
public class PlanReportConvert {
    
    public static PlanReport convert2Domain(PlanReportDO var) {
        if (var == null) {
            return null;
        }
        
        PlanReport instance = new PlanReport();
        instance.setRptNo(var.getRptNo());
        instance.setPlanNo(var.getPlanNo());
        instance.setPlanName(var.getPlanName());
        instance.setRptType(ReportTypeEnum.getByCode(var.getRptType()));
        
        if (instance.getRptType() == ReportTypeEnum.RPT001) {
            ReportDateField reportDateField = new DailyReportDateField();
            reportDateField.parse(var.getRptDate());
            instance.setRptDate(reportDateField);
        } else if (instance.getRptType() == ReportTypeEnum.RPT002){
            ReportDateField reportDateField = new MonthReportDateField();
            reportDateField.parse(var.getRptDate());
            instance.setRptDate(reportDateField);
        }
        instance.setDetail(var.getRptDtl());
        return instance;
    }
    
    public static List<PlanReportItem> convertItems2Domain(List<PlanReportItemDO> planReportItemDOList) {
        List<PlanReportItem> planReportItems = Lists.newArrayList();
        
        for (PlanReportItemDO planReportItemDO : planReportItemDOList) {
            planReportItems.add(convertItem2Domain(planReportItemDO));
        }
        return planReportItems;
    }
    
    public static PlanReportItem convertItem2Domain(PlanReportItemDO planReportItemDO) {
        if (planReportItemDO == null){
            return null;
        }
        PlanReportItem planReportItem = new PlanReportItem();
        planReportItem.setPlanNo(planReportItemDO.getPlanNo());
        planReportItem.setRptItemCode(planReportItemDO.getRptItem());
        planReportItem.setRptItemName(planReportItemDO.getRptItemName());
        planReportItem.setAstCnt(planReportItemDO.getAstCnt());
        planReportItem.setAmt(planReportItemDO.getAmt());
        planReportItem.setAccountDate(planReportItemDO.getAccountDate());
        return planReportItem;
    }
    
    public static PlanReportDO convert2DO(PlanReport var) {
        if (var == null) {
            return null;
        }
        
        PlanReportDO instance = new PlanReportDO();
        
        instance.setRptNo(var.getRptNo());
        instance.setPlanNo(var.getPlanNo());
        instance.setPlanName(var.getPlanName());
        instance.setRptDate(var.getRptDate().desc());
        instance.setRptType(var.getRptType().getCode());
        instance.setRptDtl(var.getDetail());
        
        return instance;
    }
    
    public static List<PlanReportItemDO> convertItems2Do(PlanReport planReport) {
        List<PlanReportItemDO> planReportItemDOs = Lists.newArrayList();
        
        for (PlanReportItem planReportItem : planReport.getReportItems()) {
            planReportItem.setRptNo(planReport.getRptNo());
            PlanReportItemDO planReportItemDO = new PlanReportItemDO();
            planReportItemDO.setAstCnt(planReportItem.getAstCnt());
            planReportItemDO.setRptNo(planReportItem.getRptNo());
            planReportItemDO.setPlanNo(planReportItem.getPlanNo());
            planReportItemDO.setRptItem(planReportItem.getRptItemCode());
            planReportItemDO.setRptItemName(planReportItem.getRptItemName());
            planReportItemDO.setAccountDate(planReportItem.getAccountDate());
            planReportItemDO.setAmt(planReportItem.getAmt());
            
            planReportItemDOs.add(planReportItemDO);
        }
        
        return planReportItemDOs;
    }
    
    public static PlanReport defaultRPT001(String planNo, ReportTypeEnum type, PlanReportItemDefaultDO defaultDO) {
        PlanReport instance = new PlanReport();
        instance.setRptNo(defaultDO.getRptNo());
        instance.setPlanNo(planNo);
        instance.setRptType(type);
    
        DailyReportDateField reportDateField = new DailyReportDateField();
        reportDateField.setRptDate(defaultDO.getAccountDate());
        instance.setRptDate(reportDateField);
        return instance;
    }
    
    public static List<PlanReportItem> convertDefaultItems2Domain(List<PlanReportItemDefaultDO> planReportItemDOList) {
        List<PlanReportItem> planReportItems = Lists.newArrayList();
        
        for (PlanReportItemDefaultDO planReportItemDO : planReportItemDOList) {
            planReportItems.add(convertItem2Domain(planReportItemDO));
        }
        return planReportItems;
    }
    
    public static PlanReportItem convertItem2Domain(PlanReportItemDefaultDO defaultDO) {
        if (defaultDO == null){
            return null;
        }
        PlanReportItem planReportItem = new PlanReportItem();
        planReportItem.setPlanNo(defaultDO.getPlanNo());
        planReportItem.setRptItemCode(defaultDO.getRptItem());
        planReportItem.setRptItemName(defaultDO.getRptItemName());
        planReportItem.setAstCnt(defaultDO.getAstCnt());
        planReportItem.setAmt(defaultDO.getAmt());
        planReportItem.setAccountDate(defaultDO.getAccountDate());
        return planReportItem;
    }
    
    public static List<PlanReportItemDefaultDO> initItemDos(String planNo, Date acctDate,
                                                            List<ReportField> fields, List<PlanReportItemDefaultDO> dos) {
        List<String> exist = dos.stream().map(PlanReportItemDefaultDO::getRptItem).collect(Collectors.toList());
        List<PlanReportItemDefaultDO> itemDOS = new ArrayList<>();
        for(ReportField field : fields) {
            // 已存在的统计项 不重新生成
            if (ToolUtil.isNotEmpty(exist) && exist.contains(field.getCode())){
                continue;
            }
            PlanReportItemDefaultDO itemDO = new PlanReportItemDefaultDO();
            itemDO.setAstCnt(0L);
            itemDO.setRptNo("DEFAULT");
            itemDO.setPlanNo(planNo);
            itemDO.setRptItem(field.getCode());
            itemDO.setRptItemName(field.getName());
            itemDO.setAccountDate(acctDate);
            itemDO.setAmt(BigDecimal.ZERO);
            itemDOS.add(itemDO);
        }
        return itemDOS;
    }
    
    public static void mergeItem(List<PlanReportItemDefaultDO> reportItemDefaultList, List<PlanReportItem> reportItems){
        if (ToolUtil.isEmpty(reportItemDefaultList)) {
            return;
        }
        List<PlanReportItem> defaultItems = PlanReportConvert.convertDefaultItems2Domain(reportItemDefaultList);
        for (PlanReportItem t : defaultItems) {
            // 如果同一统计项日期一致则不合并
            boolean match = reportItems.stream().allMatch(one -> {
                boolean day = true;
                if (ToolUtil.isNotEmpty(one.getAccountDate())
                    && ToolUtil.isNotEmpty(t.getAccountDate())){
                    day = one.getAccountDate().compareTo(t.getAccountDate()) == 0;
                }
                return day && ToolUtil.equals(one.getRptItemCode(), t.getRptItemCode());
            });
            if (!match) {
                reportItems.add(t);
            }
        }
    }
    
    public static void summryByItem(List<PlanReportItemDefaultDO> reportItemDefaultList, List<PlanReportItem> reportItems) {
        if (ToolUtil.isEmpty(reportItemDefaultList)) {
            return;
        }
        for (PlanReportItem item : reportItems) {
            for (PlanReportItemDefaultDO defaultDO : reportItemDefaultList){
                if (ToolUtil.equals(item.getRptItemCode(), defaultDO.getRptItem())){
                    item.setAmt(NumberUtil.add(item.getAmt(), defaultDO.getAmt()));
                    item.setAstCnt(NumberUtil.add(item.getAstCnt(), defaultDO.getAstCnt()).longValue());
                }
            }
        }
    }
}