/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.ProductDO;
import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.facade.param.prod.ProdEditParam;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author hanfei
 * @version $Id: ProductConvert.java, v 0.1 2018-01-19 下午8:27 hanfei Exp $
 */
public class ProductConvert {
    public static String defaultProdCode = "COMMON";

    public static List<LoanProduct> convertDTOs(List<ProductDO> models) {
        if (CollectionUtils.isEmpty(models)) {
            return Lists.newArrayList();
        }
        List<LoanProduct> dtos = new ArrayList<LoanProduct>(models.size());
        for (ProductDO oneDO : models) {
            LoanProduct one = convertDTO(oneDO);
            if (one == null) {
                continue;
            }
            dtos.add(one);
        }
        return dtos;
    }

    //
    public static LoanProduct convertDTO(ProductDO oneDO) {
        if (oneDO == null) {
            return null;
        }
        LoanProduct dto = new LoanProduct();
        /**
         * 权益归属机构
         */
        dto.setLenderOrgCode(oneDO.getLenderOrgCode());
        /**
         * 代理机构编码
         */
        dto.setManagerOrgCode(oneDO.getManagerOrgCode());
        /**
         * 产品名称
         */
        dto.setProdName(oneDO.getProdName());
        /**
         * 内部产品编码: 唯一组件
         */
        dto.setAbssqrProdCode(oneDO.getAbssqrProdCode());
        /**
         * 外部产品编码
         */
        dto.setProdCode(oneDO.getProdCode());
        /**
         * 审批状态
         */
        dto.setApproveStatus(ApproveStatusEnum.getEnum(oneDO.getApproveStatus()));

        return dto;
    }

    public static ProductDO prodEditParam2DO(ProdEditParam param) {
        if (param == null) {
            return null;
        }

        ProductDO result = new ProductDO();
        //prodCode 外部产品码
        if (StringUtils.isBlank(param.getProdCode())) {
            result.setProdCode(AbssqrConstants.DUMMY_PROD_CODE);
        } else {
            result.setProdCode(param.getProdCode());
        }

        //prodName 产品名
        result.setProdName(param.getProdName());
        //approveStatus 审批状态
        if (param.getApproveStatus() != null) {
            result.setApproveStatus(param.getApproveStatus());
        }
        //lenderOrgCode 贷款的实际出资机构
        result.setLenderOrgCode(param.getLenderOrgCode());
        //abssqrProdCode 内部产品码
        result.setAbssqrProdCode(param.getAbssqrProdCode());
        //managerOrgCode 贷款的管理机构
        result.setManagerOrgCode(param.getManagerOrgCode());
        return result;

    }

}
