/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.TrCtrTaskDO;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.TrCtrTask;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;

import java.util.List;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.convertor: TrCtrTaskConvert.java, v 0.1 2019-06-18 10:33 bangis.wangdf Exp $
 */
public class TrCtrTaskConvert {
    
    public static List<TrCtrTaskDO> convert2DOs(List<TrCtrTask> trCtrTasks) {
        List<TrCtrTaskDO> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(trCtrTasks)) {
            for (TrCtrTask trCtrTask : trCtrTasks) {
                if (trCtrTask.getPreTrCnt() > 0) {
                    result.add(convert2DO(trCtrTask));
                }

            }
        }
        return result;
    }

    public static TrCtrTaskDO convert2DO(TrCtrTask param) {
        if (null == param) {
            return null;
        }
        TrCtrTaskDO result = new TrCtrTaskDO();
        //preTrAmt 预转让金额
        result.setPreTrAmt(param.getPreTrAmt());
        //preTrCnt 预转让笔试
        result.setPreTrCnt(param.getPreTrCnt());
        //batchNo 批次号
        result.setBatchNo(param.getBatchNo());
        //packCode 资产包编号
        result.setPackCode(param.getPackCode());
        //资产包类型
        result.setPackType(param.getPackType().getCode());
        //tfrDate 转让日期
        result.setTfrDate(param.getTfrDate());
        result.setStatus(param.getTaskStatus().getCode());
        result.setExecTms(param.getExecTimes()==null?0:param.getExecTimes());
        result.setManagerOrgCode(param.getManagerOrgCode());
        return result;
    }
    
    public static List<TrCtrTask> convert2Domains(List<TrCtrTaskDO> trCtrTasks) {
        List<TrCtrTask> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(trCtrTasks)) {
            for (TrCtrTaskDO trCtrTask : trCtrTasks) {
                if (trCtrTask.getPreTrCnt() > 0) {
                    result.add(convert2Domain(trCtrTask));
                }
            }
        }
        return result;
    }
    
    public static TrCtrTask convert2Domain(TrCtrTaskDO param) {
        if (null == param) {
            return null;
        }
        TrCtrTask result = new TrCtrTask();
        result.setId(param.getId());
        //preTrAmt 预转让金额
        result.setPreTrAmt(param.getPreTrAmt());
        //preTrCnt 预转让笔试
        result.setPreTrCnt(param.getPreTrCnt());
        //batchNo 批次号
        result.setBatchNo(param.getBatchNo());
        //packCode 资产包编号
        result.setPackCode(param.getPackCode());
        //资产包类型
        result.setPackType(AstPackTradeTypeEnum.getByCode(param.getPackType()));
        //tfrDate 转让日期
        result.setTfrDate(param.getTfrDate());
        result.setTaskStatus(TaskStatusEnum.getByCode(param.getStatus()));
        result.setExecTimes(param.getExecTms());
        result.setManagerOrgCode(param.getManagerOrgCode());
        return result;
    }
}
