/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.TrTaskDO;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.TrTask;
import org.apache.commons.collections.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 *
 * @author hanfei
 * @version $Id: TrTaskConvert.java, v 0.1 2018-01-20 上午12:01 hanfei Exp $ 
 */
public class TrTaskConvert {

    public static List<TrTaskDO> convertDOs(List<TrTask> trTasks) {
        if (CollectionUtils.isEmpty(trTasks)) {
            return null;
        }
        List<TrTaskDO> taskDOS = new ArrayList<TrTaskDO>(trTasks.size());
        for(TrTask trTask : trTasks){
            TrTaskDO one = convertDO(trTask);
            if (one == null) {
                continue;
            }
            taskDOS.add(one);
        }
        return taskDOS;
    }

    //
    public static TrTaskDO convertDO(TrTask trTask) {
        if (trTask == null) {
            return null;
        }
        TrTaskDO trTaskDO = new TrTaskDO();

        /**
         * tfrDate 转让日期.
         */
        trTaskDO.setTfrDate(trTask.getTfrDt());
        /**
         * astEndDate 资产到期日.
         */
        trTaskDO.setAstEndDate(trTask.getAstDueDt());
        /**
         * lastExecTime 最近执行时间.
         */
        trTaskDO.setLastExecTime(trTask.getLastExecTime());
        trTaskDO.setNextExecTime(trTask.getNextExecTime());
        /**
         * tfrAmt 资产转让最终值.
         */
        trTaskDO.setTfrAmt(trTask.getTfrAmt());
        /**
         * tfrPrinBal 转让本金余额.
         */
        trTaskDO.setTfrPrinBal(trTask.getTfrPrinBal());

        /**
         * astId  账单id.
         */
        trTaskDO.setAstNo(trTask.getAstNo());
        /**
         * seqNo 流水号.
         */
        trTaskDO.setSeqNo(trTask.getSeqNo());
        /**
         * tfrOut 资产转出机构.
         */
        trTaskDO.setTfrOut(trTask.getTfrOut());

        /**
         * batchNo 批次号-防止异常.
         */
        trTaskDO.setBatchNo(trTask.getBatchNo());
        /**
         * tfrPlan 转让计划.
         */
        trTaskDO.setTfrPack(trTask.getTfrPack());
        /**
         * lastHost 最近执行服务器.
         */
        trTaskDO.setLastHost(trTask.getLastHost());
        /**
         * taskStatus 任务状态.
         */
        trTaskDO.setTaskStatus(trTask.getTaskStatus().getCode());
        /**
         * execTimes 执行次数.
         */
        trTaskDO.setExecTimes(trTask.getExecTimes());
        trTaskDO.setTermNo(trTask.getTermNo());
        trTaskDO.setTfrIntrBal(trTask.getTfrIntrBal());
        trTaskDO.setTfrPenalFeeBal(trTask.getTfrPenalFeeBal());
        trTaskDO.setRiskCl(trTask.getRiskCl());
        trTaskDO.setIntrRate(trTask.getIntrRate());
        trTaskDO.setPrinRate(trTask.getPrinRate());
        trTaskDO.setPenalFeeRate(trTask.getPenalFeeRate());
        trTaskDO.setPackType(Optional.ofNullable(trTask.getPackType()).map(AstPackTradeTypeEnum::getCode).orElse(null));
        return trTaskDO;
    }

    public static List<TrTask> convertDTOs(List<TrTaskDO> models) {
        if (CollectionUtils.isEmpty(models)) {
            return null;
        }
        List<TrTask> dtos = new ArrayList<TrTask>(models.size());
        for(TrTaskDO oneDO : models){
            TrTask one = convertDTO(oneDO);
            if (one == null) {
                continue;
            }
            dtos.add(one);
        }
        return dtos;
    }

    //
    public static TrTask convertDTO(TrTaskDO entity) {
        if (entity == null) {
            return null;
        }
        TrTask model = new TrTask();
        /**
         * tfrDate 转让日期.
         */
        model.setTfrDt(entity.getTfrDate());
        /**
         * astEndDate 资产到期日.
         */
        model.setAstDueDt(entity.getAstEndDate());
        /**
         * lastExecTime 最近执行时间.
         */
        model.setLastExecTime(entity.getLastExecTime());
        model.setNextExecTime(entity.getNextExecTime());
        /**
         * tfrAmt 资产转让最终值.
         */
        model.setTfrAmt(entity.getTfrAmt());
        /**
         * tfrPrinBal 转让本金余额.
         */
        model.setTfrPrinBal(entity.getTfrPrinBal());
        /**
         * memo memo.
         */
//        model.setMemo(entity.get);
        /**
         * astId  账单id.
         */
        model.setAstNo(entity.getAstNo());
        /**
         * seqNo 流水号.
         */
        model.setSeqNo(entity.getSeqNo());
        /**
         * tfrOut 资产转出机构.
         */
        model.setTfrOut(entity.getTfrOut());
      
        /**
         * batchNo 批次号-防止异常.
         */
        model.setBatchNo(entity.getBatchNo());
        /**
         * tfrPlan 转让计划.
         */
        model.setTfrPack(entity.getTfrPack());
        /**
         * lastHost 最近执行服务器.
         */
        model.setLastHost(entity.getLastHost());
        /**
         * taskStatus 任务状态.
         */
        model.setTaskStatus(TaskStatusEnum.getByCode(entity.getTaskStatus()));
        /**
         * execTimes 执行次数.
         */
        model.setExecTimes(entity.getExecTimes());

        model.setId(entity.getId());

        model.setTermNo(entity.getTermNo());
        // 利息余额
        model.setTfrIntrBal(entity.getTfrIntrBal());
        // 其他费用余额
        model.setTfrPenalFeeBal(entity.getTfrPenalFeeBal());
        // 五级分类
        model.setRiskCl(entity.getRiskCl());
        //
        model.setIntrRate(entity.getIntrRate());
        model.setPrinRate(entity.getPrinRate());
        model.setPenalFeeRate(entity.getPenalFeeRate());
        model.setPackType(AstPackTradeTypeEnum.getByCode(entity.getPackType()));
        return model;
    }
}
