/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.domain.ast;

import com.abssqr.plat.common.facade.base.CreationAwareEntity;
import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 贷款资产分期
 *
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.asset: LoanInstal.java, v 0.1 2017-12-12 下午2:49 guozhenhua Exp $
 */
public class LoanInstal extends CreationAwareEntity {
    
    /**
     * 贷款资产日期
     */
    final private AstPeriod           astPeriod   = new AstPeriod();
    /**
     * 资产机构信息
     */
    final private AstOrg              astOrg      = new AstOrg();
    /**
     * 资产ID
     */
    private       String              astNo;
    /**
     * 用户ID
     */
    private       String              userId;
    /**
     * 外部传入的资产唯一编号
     */
    private       String              outAstNo;
    /**
     * 合同编号
     */
    private       String              contNo;
    /**
     * 内部产品编码
     */
    private       String              prodNo;
    /**
     * 外部产品产品
     */
    private       String              outProdNo;
    /**
     * 所属资产包
     */
    private       String              ownPack;
    /**
     * 期次号
     */
    private       int                 termNo;
    /**
     * 到期利息金额
     */
    private       Money               dueIntrAmt  = new Money(0);
    /**
     * 到期费用金额
     */
    private       Money               dueFeeAmt   = new Money(0);
    /**
     * 悬挂利息金额
     */
    private       Money               hangIntrBal = new Money(0);
    /**
     * 悬挂利息金额
     */
    private       Money               hangIntrBak = new Money(0);
    /**
     * 已付悬挂利息金额
     */
    private       Money               paidHangIntrAmt = new Money(0);

    /**
     * 成本
     */
    private Money               cost        = new Money(0);
    /**
     * 余额
     */
    private DebtMoney           bal         = new DebtMoney();
    /**
     * 已还金额
     */
    private DebtMoney           paidAmt     = new DebtMoney();
    /**
     * 减免金额
     */
    private DebtMoney           rmtAmt      = new DebtMoney();
    /**
     * 五矿实收已还金额
     */
    private DebtMoney           paidFundAmt = new DebtMoney();
    /**
     * 资产状态
     */
    private LoanStatusEnum      status;
    /**
     * 逾期天数
     */
    private int                 ovdDays;
    /**
     * 最近数据发生日(yyyyMMdd)
     */
    private Date                lastUpdateDate;
    /**
     * 扩展信息
     */
    private Map<String, String> extInf      = new HashMap<String, String>();
    
    private Loan refLoan;
    
    public Loan getRefLoan() {
        return refLoan;
    }
    
    public void setRefLoan(Loan refLoan) {
        this.refLoan = refLoan;
    }
    
    public LoanInstal() {
        super(true);
    }
    
    public LoanInstal(boolean newModel) {
        super(newModel);
    }
    
    public void checkSelf(Date accountDate) {
        if (this.status == LoanStatusEnum.REVERSE) {
            return;
        }
        
        bal.check(MessageUtil.formatMsg("[loanCheck]分期[{0}-{1}]余额[{2}]校验失败", astNo, termNo, bal));
        paidAmt.check(MessageUtil.formatMsg("[loanCheck]分期[{0}-{1}]已还[{2}]校验失败", astNo, termNo, paidAmt));
        astPeriod.checkSelf();
        
        if (status == LoanStatusEnum.CLEAR) {
            bal.checkZero(MessageUtil.formatMsg("[loanCheck][{0}-{1}]状态校验失败，已结清分期，余额必须为0", astNo, termNo));
            VarChecker.checkNotNull(astPeriod.getClearDate(), "[loanCheck][{0}-{1}]已结清分期，结清日期不能为空"
                    , astNo, termNo);
        }
        if(refLoan.getLoanRisk().getWtfStatus()!=YNEnum.N) {
            if (DateTimeUtil.calcDayDiff(accountDate, astPeriod.getEndDate()) > 0 && bal.getSum().isGreaterThanZero()) {
                VarChecker.checkArgument(status == LoanStatusEnum.OVERDUE,
                        "[loanCheck][{0}-{1}]状态校验失败，分期应该为逾期状态[{2}]", astNo, termNo, this.outAstNo);
            }

            if (DateTimeUtil.calcDayDiff(accountDate, astPeriod.getEndDate()) <= 0) {
                VarChecker.checkArgument(
                        status == LoanStatusEnum.NORMAL || status == LoanStatusEnum.CLEAR,
                        "[loanCheck][{0}-{1}]未到期资产[{2}]，状态必须为结清或者正常[{3}]", astNo, termNo, accountDate, this.outAstNo);
            }
        }
    }

    public LoanInstal clone(int termNo) {
        LoanInstal loanInstal = new LoanInstal();
        cloneInstalData(loanInstal, termNo);
        
        //        loanInstal.setHangIntrBal(this.getHangIntrBal().clone());
        //        loanInstal.setCost(this.getCost().clone());
        //        loanInstal.setDueIntrAmt(this.getDueIntrAmt().clone());
        //        loanInstal.setDueFeeAmt(this.getDueFeeAmt().clone());
        //        loanInstal.setBal(this.getBal().cloneAll());
        //        loanInstal.setPaidAmt(this.getPaidAmt().cloneAll());
        //        loanInstal.setRmtAmt(this.getRmtAmt().cloneAll());
        //        loanInstal.setPaidFundAmt(this.getPaidFundAmt().cloneAll());
        return loanInstal;
    }
    
    protected void cloneInstalData(LoanInstal loanInstal, int termNo) {
        loanInstal.setAstNo(this.getAstNo());
        loanInstal.setUserId(this.getUserId());
        loanInstal.setOutAstNo(this.getOutAstNo());
        loanInstal.setContNo(this.getContNo());
        loanInstal.setProdNo(this.getProdNo());
        loanInstal.setOutProdNo(this.getOutProdNo());
        loanInstal.setOwnPack(this.getOwnPack());
        loanInstal.setTermNo(termNo);
    
        loanInstal.setStatus(this.getStatus());
        loanInstal.setOvdDays(this.getOvdDays());
        loanInstal.setLastUpdateDate(this.getLastUpdateDate());
    
        loanInstal.getAstPeriod().cloneTo(this.getAstPeriod());
        loanInstal.getAstOrg().cloneTo(this.getAstOrg());
    }
    
    public Money getTotalAccurAmt(DebtMoneyFieldEnum amtType) {
        return this.bal.getByFieldType(amtType).add(this.paidAmt.getByFieldType(amtType))
                       .add(this.rmtAmt.getByFieldType(amtType));
    }
    
    public String getUserId() {
        return userId;
    }
    
    public void setUserId(String userId) {
        this.userId = userId;
    }
    
    public AstPeriod getAstPeriod() {
        return astPeriod;
    }
    
    public AstOrg getAstOrg() {
        return astOrg;
    }
    
    public String getAstNo() {
        return astNo;
    }
    
    public void setAstNo(String astNo) {
        this.astNo = astNo;
    }
    
    public String getOutAstNo() {
        return outAstNo;
    }
    
    public void setOutAstNo(String outAstNo) {
        this.outAstNo = outAstNo;
    }
    
    public String getContNo() {
        return contNo;
    }
    
    public void setContNo(String contNo) {
        this.contNo = contNo;
    }
    
    public String getProdNo() {
        return prodNo;
    }
    
    public void setProdNo(String prodNo) {
        this.prodNo = prodNo;
    }
    
    public String getOutProdNo() {
        return outProdNo;
    }
    
    public void setOutProdNo(String outProdNo) {
        this.outProdNo = outProdNo;
    }
    
    public String getOwnPack() {
        return ownPack;
    }
    
    public void setOwnPack(String ownPack) {
        this.ownPack = ownPack;
    }
    
    public int getTermNo() {
        return termNo;
    }
    
    public void setTermNo(int termNo) {
        this.termNo = termNo;
    }
    
    public Money getDueIntrAmt() {
        return dueIntrAmt;
    }
    
    public void setDueIntrAmt(Money dueIntrAmt) {
        this.dueIntrAmt = dueIntrAmt;
    }
    
    public Money getDueFeeAmt() {
        return dueFeeAmt;
    }
    
    public void setDueFeeAmt(Money dueFeeAmt) {
        this.dueFeeAmt = dueFeeAmt;
    }
    
    public DebtMoney getBal() {
        return bal;
    }
    
    public void setBal(DebtMoney bal) {
        this.bal = bal;
    }
    
    public DebtMoney getPaidAmt() {
        return paidAmt;
    }
    
    public void setPaidAmt(DebtMoney paidAmt) {
        this.paidAmt = paidAmt;
    }
    
    public DebtMoney getRmtAmt() {
        return rmtAmt;
    }
    
    public void setRmtAmt(DebtMoney rmtAmt) {
        this.rmtAmt = rmtAmt;
    }
    
    public LoanStatusEnum getStatus() {
        return status;
    }
    
    public void setStatus(LoanStatusEnum status) {
        this.status = status;
    }
    
    public int getOvdDays() {
        return ovdDays;
    }
    
    public void setOvdDays(int ovdDays) {
        this.ovdDays = ovdDays;
    }
    
    public Date getLastUpdateDate() {
        return lastUpdateDate;
    }
    
    public void setLastUpdateDate(Date lastUpdateDate) {
        this.lastUpdateDate = lastUpdateDate;
    }
    
    public Map<String, String> getExtInf() {
        return extInf;
    }
    
    public void setExtInf(Map<String, String> extInf) {
        this.extInf = extInf;
    }
    
    public Money getHangIntrBal() {
        return hangIntrBal;
    }
    
    public void setHangIntrBal(Money hangIntrBal) {
        this.hangIntrBal = hangIntrBal;
    }
    
    public Money getCost() {
        return cost;
    }
    
    public void setCost(Money cost) {
        this.cost = cost;
    }
    
    public DebtMoney getPaidFundAmt() {
        return paidFundAmt;
    }
    
    public void setPaidFundAmt(DebtMoney paidFundAmt) {
        this.paidFundAmt = paidFundAmt;
    }

    public Money getPaidHangIntrAmt() {
        return paidHangIntrAmt;
    }

    public void setPaidHangIntrAmt(Money paidHangIntrAmt) {
        this.paidHangIntrAmt = paidHangIntrAmt;
    }
    
    public Money getHangIntrBak() {
        return hangIntrBak;
    }
    
    public void setHangIntrBak(Money hangIntrBak) {
        this.hangIntrBak = hangIntrBak;
    }
}
