/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.domain.ast;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.LoanAstStatusEnum;
import com.abssqr.plat.common.facade.enums.ProfitTypeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.domain.ast: LoanInstalDomain.java, v 0.1 2019-11-13 下午15:20 zhenxuan.luo Exp $ 
 */
public class LoanInstalDomain extends LoanInstal {
    
    /**
     * 所属机构资产
     */
    private List<LoanAst> loanAsts = Lists.newArrayList();
    
    /**
     * 由当期分期管理的机构收益：从当前分期摊出
     */
    private List<LoanAstProfit> sourcedProfits = Lists.newArrayList();
    
    public LoanInstalDomain() {
        super(true);
    }
    
    public LoanInstalDomain(boolean newModel) {
        super(newModel);
    }
    
    public LoanAstGroup filterLoanAst(String ownOrg) {
        LoanAstGroup loanAstGroup = new LoanAstGroup();
        loanAstGroup.setMaxTerm(this.getTermNo());
        loanAstGroup.setMinTerm(this.getTermNo());
        loanAstGroup.getRefInstalMap().put(this.getTermNo(), this);
        for (LoanAst loanAst : loanAsts) {
            if (loanAst.getAstStatus() != LoanAstStatusEnum.NORMAL) {
                continue;
            }
            if (StringUtils.equals(ownOrg, loanAst.getOwnOrg())) {
                loanAstGroup.getLoanAsts().add(loanAst);
                loanAstGroup.getPrinBal().addTo(loanAst.getPrinBal());
            } else {
                loanAstGroup.setAllAst(false);
            }
        }
        
        return loanAstGroup;
    }
    
    public LoanAstGroup buildAstGroup() {
        LoanAstGroup loanAstGroup = new LoanAstGroup();
        loanAstGroup.setMaxTerm(this.getTermNo());
        loanAstGroup.setMinTerm(this.getTermNo());
        loanAstGroup.getRefInstalMap().put(this.getTermNo(), this);
        for (LoanAst loanAst : loanAsts) {
            loanAstGroup.getLoanAsts().add(loanAst);
            loanAstGroup.getPrinBal().add(loanAst.getPrinBal());
        }
        
        return loanAstGroup;
    }
    
    public LoanAst addLoanAst(LoanAst loanAst) {
        if (loanAst.getAstStatus() == LoanAstStatusEnum.NORMAL) {
            LoanAst existAst = filterLoanAst(loanAst.getOwnOrg(), loanAst.getInRule());
            if (null != existAst) {
                throw new AbssqrBizException(MessageUtil.formatMsg("同一分期下，出现两笔转入规则相同且同时有效的机构资产[{0}]", existAst),
                        CommonErrorCodeEnum.INVALID_INTERNAL_MODEL, false);
            }
        }
        this.loanAsts.add(loanAst);
        return loanAst;
    }
    
    public LoanAst filterLoanAst(String ownOrg, String ruleNo) {
        LoanAst ret = null;
        
        for (LoanAst loanAst : loanAsts) {
            if (loanAst.getAstStatus() != LoanAstStatusEnum.NORMAL) {
                continue;
            }
            // 1. 都为空
            boolean isInRuleSame = StringUtils.isBlank(ruleNo) && StringUtils.isBlank(loanAst.getInRule());
            isInRuleSame = isInRuleSame || StringUtils.equals(ruleNo, loanAst.getInRule());
            if (StringUtils.equals(ownOrg, loanAst.getOwnOrg()) && isInRuleSame) {
                if (ret == null) {
                    ret = loanAst;
                } else {
                    throw new AbssqrBizException(MessageUtil.formatMsg("同一分期下，出现两笔转入规则相同且同时有效的机构资产[{0}]", ret),
                            CommonErrorCodeEnum.INVALID_INTERNAL_MODEL, false);
                }
            }
        }
        
        return ret;
    }
    
    @Override
    public void checkSelf(Date accountDate) {
        super.checkSelf(accountDate);
        
        DebtMoney allAstBal = new DebtMoney();
        DebtMoney allPaid = new DebtMoney();
        DebtMoney allFund = new DebtMoney();
        DebtMoney allEntrySum = new DebtMoney();
        DebtMoney allEntry = new DebtMoney();
        
        for (LoanAst loanAst : loanAsts) {
            loanAst.checkSelf();
            allAstBal.addByFieldType(DebtMoneyFieldEnum.PRIN, loanAst.getPrinBal());
            allPaid.addByFieldType(DebtMoneyFieldEnum.PRIN, loanAst.getPaidPrinAmt());
            allFund.addByFieldType(DebtMoneyFieldEnum.PRIN, loanAst.getPaidPrinAmt());
        }
        
        for (LoanAstProfit loanAstProfit : sourcedProfits) {
            allAstBal.addAll(loanAstProfit.getBal());
            allPaid.addAll(loanAstProfit.getPaidAmt());
            allFund.addAll(loanAstProfit.getFundAmt());
            allEntrySum.addAll(loanAstProfit.getRmtAmt());

            if (loanAstProfit.getProfitType() == ProfitTypeEnum.OWN) {
                allEntry.addAll(loanAstProfit.getEntryAmt());
            }
        }
        allEntrySum.addAll(allAstBal, DebtMoneyFieldEnum.PRIN).addAll(allPaid, DebtMoneyFieldEnum.PRIN);
        
        VarChecker.checkEquals(allAstBal, this.getBal(), "[loanCheck]分期[{0}-{1}]余额[{2}]和分期机构资产[{3}]汇总不相等",
                this.getAstNo(), this.getTermNo(), this.getBal(), allAstBal);
        VarChecker.checkEquals(allPaid, this.getPaidAmt(), "[loanCheck]分期[{0}-{1}]已还[{2}]和分期机构资产[{3}]汇总不相等",
                this.getAstNo(), this.getTermNo(), this.getPaidAmt(), allPaid);
        VarChecker.checkEquals(allFund, this.getPaidFundAmt(), "[loanCheck]分期[{0}-{1}]实还[{2}]和分期机构资产[{3}]汇总不相等",
                this.getAstNo(), this.getTermNo(), this.getPaidFundAmt(), allFund);
        DebtMoney instalEntry = this.getRmtAmt().cloneAll().addAll(this.getPaidAmt(), DebtMoneyFieldEnum.PRIN)
                                        .addAll(this.getBal(), DebtMoneyFieldEnum.PRIN);
        VarChecker.checkEquals(allEntry, instalEntry, "[loanCheck]分期[{0}-{1}]总入账[{2}]和分期机构资产[{3}]汇总不相等",
                this.getAstNo(), this.getTermNo(), instalEntry, allEntry);
        VarChecker.checkEquals(allEntry, allEntrySum, "[loanCheck]分期[{0}-{1}]机构资产总入账[{2}]和分期机构资产分项金额[{3}]汇总不相等",
                this.getAstNo(), this.getTermNo(), allEntry, allEntrySum);
        
    }
    
    @Override
    public LoanInstalDomain clone(int termNo) {
        LoanInstalDomain instalDomain = new LoanInstalDomain();
        cloneInstalData(instalDomain, termNo);
        return instalDomain;
    }
    
    public Map<String, LoanAst> getLoanAstMap() {
        return this.loanAsts.stream().collect(Collectors.toMap(LoanAst::getLoanAstNo, item -> item));
    }
    
    public LoanAst getLoanAstByAstNo(String astNo) {
        Map<String, LoanAst> loanAstMap = this.loanAsts.stream()
                                                  .collect(Collectors.toMap(LoanAst::getLoanAstNo, item -> item));
        return loanAstMap.get(astNo);
    }
    
    public Map<String, BigDecimal> getPrinDist() {
        Map<String, BigDecimal> ret = Maps.newHashMap();
        
        // 不做机构合并，可能存在一个机构持有两个机构资产
        for (LoanAst loanAst : loanAsts) {
            if (loanAst.getAstStatus() != LoanAstStatusEnum.NORMAL) {
                continue;
            }
            ret.put(loanAst.getLoanAstNo(), loanAst.getPrinBal().getAmount());
        }
        
        return ret;
    }
    
    public List<LoanAst> getLoanAsts() {
        return loanAsts;
    }
    
    public void setLoanAsts(List<LoanAst> loanAsts) {
        this.loanAsts = loanAsts;
    }
    
    public List<LoanAstProfit> getSourcedProfits() {
        return sourcedProfits;
    }
    
}