/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.domain.plan;

import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.model.domain.loanTrans.ITransDtl;
import com.abssqr.plat.common.model.domain.loanTrans.TransInfoTemplate;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.MessageUtil;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 资产变更明细
 *
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.domain.loanTrans: LoanTransDetail.java, v 0.1 2017-12-12 下午8:20 guozhenhua Exp $
 */
public class PlanAcctTitleTransDetail implements ITransDtl {
    
    /**
     * 贷款资产
     */
    private PlanAcctTitle planAcctTitle;
    
    private PlanAcctTitleLog activeLog;
    
    private boolean isUpdate = false;
    
    private boolean isAdd = false;
    
    private boolean isSub = false;
    
    private PlanAcctTitleTransDetail() {
    
    }
    
    /**
     * 构建科目交易流水
     *
     * @param planAcctTitle
     * @param transInfo
     * @return
     */
    public static PlanAcctTitleTransDetail createTxnDtl(PlanAcctTitle planAcctTitle, TransInfoTemplate transInfo, boolean isReal) {
        
        // 1. 构造资产交易明细
        PlanAcctTitleTransDetail transDetail = new PlanAcctTitleTransDetail();
        transDetail.planAcctTitle = planAcctTitle;
        
        PlanAcctTitleLog planAcctTitleLog = new PlanAcctTitleLog();
        planAcctTitleLog.setRefNo(planAcctTitle.getRefNo());
        planAcctTitleLog.setPlanNo(planAcctTitle.getPlanNo());
        planAcctTitleLog.setTitleNo(planAcctTitle.getTitleNo());
        planAcctTitleLog.setTitleName(planAcctTitle.getTitleName());
        planAcctTitleLog.setTitleType(planAcctTitle.getTitleType());
//        planAcctTitleLog.setCalcType(planAcctTitle.getCalcType());
        planAcctTitleLog.setBfBal(planAcctTitle.getBal().clone());
        planAcctTitleLog.setBizNo(transInfo.getBizNo());
        // 避免在模拟计息时，也去数据库拿Sequence
        if (isReal) {
            planAcctTitleLog.setSeqNo(SequenceUtil.genSeqNo(transInfo.getEventCode().getSceneCode()));
        }
        planAcctTitleLog.setAccountDate(transInfo.getAccountDate());
        planAcctTitleLog.setEventCode(transInfo.getEventCode());
        transDetail.activeLog = planAcctTitleLog;
        
        return transDetail;
    }
    
    @Override
    public void processBeforeUpdate() {
        // 将交易后信息填入交易流水
        activeLog.setAfBal(planAcctTitle.getBal().clone());
    }
    
    public void rcepAmt(Money amt, Money taxAmt) {
        if (isSub) {
            throw new AbssqrBizException(MessageUtil.formatMsg("科目[{0}]已经在进行减操作，无法进行加操作", this.planAcctTitle),
                    CommonErrorCodeEnum.INVALID_INTERNAL_MODEL);
        }
        isUpdate = true;
        isAdd = true;
        planAcctTitle.setRecpAmt(amt.add(planAcctTitle.getRecpAmt()));
        planAcctTitle.setRecpVATax(taxAmt.add(planAcctTitle.getRecpVATax()));
        planAcctTitle.setBal(planAcctTitle.getBal().add(amt));
        activeLog.setAcctAmt(activeLog.getAcctAmt().add(amt));
        activeLog.setvTaxAmt(activeLog.getvTaxAmt().add(taxAmt));
    }
    
    public void payAmt(Money amt, Money taxAmt) {
        if (isAdd) {
            throw new AbssqrBizException(MessageUtil.formatMsg("科目[{0}]已经在进行加操作，无法进行减操作", this.planAcctTitle),
                    CommonErrorCodeEnum.INVALID_INTERNAL_MODEL);
        }
        isUpdate = true;
        isSub = true;
        planAcctTitle.pay(amt);
        planAcctTitle.payVTax(taxAmt);
        activeLog.setAcctAmt(activeLog.getAcctAmt().add(amt));
        activeLog.setvTaxAmt(activeLog.getvTaxAmt().add(taxAmt));
    }
    
    public void entry(BigDecimal addTotleAmt) {
        isAdd = true;
        isUpdate = true;
        Money transAmt = planAcctTitle.entry(addTotleAmt);
        activeLog.setAcctAmt(activeLog.getAcctAmt().add(transAmt));
    }
    
    public void setSettleDate(Date settleDate) {
        activeLog.setAccountDate(settleDate);
        planAcctTitle.setLastSettleDate(settleDate);
    }
    
    public boolean isUpdate() {
        return isUpdate;
    }
    
    public PlanAcctTitle getPlanAcctTitle() {
        return planAcctTitle;
    }
    
    public PlanAcctTitleLog getActiveLog() {
        return activeLog;
    }
}
