/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.org.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.OrganizationDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.OrganizationDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.OrgListPage;
import com.abssqr.plat.common.facade.model.org.Organization;
import com.abssqr.plat.common.facade.param.org.OrgEditParam;
import com.abssqr.plat.common.facade.param.org.OrgPageQryParam;
import com.abssqr.plat.common.model.convertor.OrganizationConvert;
import com.abssqr.plat.common.model.repo.org.OrgRepository;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.Set;

/**
 * @author yaphet
 * @version com.abssqr.plat.common.model.repo.org.impl: OrgRepositoryImpl.java, v 0.1 2018-01-21 下午3:52 yaphet Exp $
 */
@Component
public class OrgRepositoryImpl implements OrgRepository {
    @Autowired
    private OrganizationDAO organizationDAO;

    @Override
    public String addOrUpdate(OrgEditParam organization) {
        VarChecker.checkNotNull(organization.getOrgCode());
        OrganizationDO organizationDO = OrganizationConvert.orgEditParam2Do(organization);
        //数据中不能有同名机构
        Integer orgCnt = organizationDAO.getSameNameOrgCnt(organizationDO);
        VarChecker.checkEquals(orgCnt, 0, "存在同名机构");
        OrganizationDO aDo = organizationDAO.getByOrgCode(organizationDO.getOrgCode());
        if (aDo == null) {
            organizationDAO.insert(organizationDO);
        } else {
            organizationDAO.updateByOrgCode(organizationDO);
        }
        return organizationDO.getOrgCode();
    }

    /**
     * @param orgCodes
     * @return
     */
    @Override
    public Map<String, String> getOrgNames(Set<String> orgCodes) {
        return organizationDAO.getOrgNames(Lists.newArrayList(orgCodes));
    }

    @Override
    public Organization getOrgByCode(String orgCode) {
        OrganizationDO organizationDO = organizationDAO.getByOrgCode(orgCode);
        Organization organization = OrganizationConvert.do2Organization(organizationDO);
        return organization;
    }


    /**
     * 分页查询机构列表
     *
     * @param orgListPage
     * @return
     */
    @Override
    public OrgListPage getOrgList(OrgPageQryParam orgListPage) {

        OrgListPage page = new OrgListPage();
        page.setOrgAbbreviation(orgListPage.getOrgAbbr());
        page.setLimit(orgListPage.getPageSize());
        page.setCurrPageNo(orgListPage.getPageNum());

        return organizationDAO.queryOrgListForPage(page);
    }

}
