/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.repo.plan;

import com.abssqr.plat.common.dal.mysql.auto.paging.PlanCapitalOrderPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.CapitalOrderAmt;
import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;

import java.util.Date;
import java.util.List;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.repo.plan: PlanCapitalOrderRepo.java, v 0.1 2019-05-28 17:22 zhenxuan.luo Exp $
 */
public interface PlanCapitalOrderRepo {
    
    /**
     * 新增
     *
     * @param entity
     * @return
     */
    int insert(PlanCapitalOrder entity);
    
    /**
     * 批量插入表
     *
     * @param list
     * @return
     */
    int insertBatch(List<PlanCapitalOrder> list);
    
    /**
     * 新增历史表
     *
     * @param entity
     * @return
     */
    int insertHis(PlanCapitalOrder entity);
    
    /**
     * 更新状态
     *
     * @param planCapitalOrder
     * @return
     */
    int updateProcess(PlanCapitalOrder planCapitalOrder);
    
    /**
     * 删除订单
     *
     * @param seqNo
     * @param planNo
     * @return
     */
    int deleteBySeqno(String seqNo, String planNo);
    
    /**
     *
     * @param seqNo
     * @return
     */
    PlanCapitalOrder lockBySeqNo(String seqNo);
    
    /**
     *
     * @param accountDate
     * @param planNo
     * @param transType
     * @return
     */
    PlanCapitalOrder lockActiveByPlanNo(Date accountDate, String planNo, PlanCapitalTransTypeEnum transType);
    
    /**
     * 根据 计划编号查询
     * @param planNo
     * @return
     */
    List<PlanCapitalOrder> getInitListByPlanNo(String planNo);
    
    /**
     * 根据条件判断当前支付单数量
     * @return
     */
    Long getActiveCntByCondition(Date accountDate, String planNo, PlanCapitalTransTypeEnum transType,
                                 List<TaskStatusEnum> statusEnums);
    
    /**
     *
     * @param planCapitalOrder
     * @return
     */
    PlanCapitalOrderPage getCapitalOrderByStatus(PlanCapitalOrderPage planCapitalOrder);
    
    /**
     * 失败的交易重新发起
     * @param seqNo
     */
    void restoreCapitalOrder(Date nextExecTime, String seqNo);
    
    /**
     * 查询执行次数较多的交易单
     * @param accountDate
     * @param execTimes
     * @param status    TaskStatusEnum
     * @return
     */
    List<PlanCapitalOrder> queryFailCapitalOrder(Date accountDate, Long execTimes, List<String> status);
    
    /**
     * desc:查询失败数量.<br/>
     * @param execTimes execTimes
     * @return Long
     */
    Long getFailCount(Long execTimes);
    
    /**
     * 根据外部交易编码查询交易单用来做mock数据
     * @param noList
     * @return
     */
    List<PlanCapitalOrder> queryOrderListByOutAcptNo(TaskStatusEnum status, List<String> noList);
    
    /**
     * 根据计划编号查询支付金额和交易金额
     * @param accountDate 会计日
     * @param planNo 计划编号
     * @param planCapitalTransTypeEnum 计划资金交易类型
     * @return CapitalOrderAmt
     */
    CapitalOrderAmt getHisAmtByPlanNo(Date accountDate, String planNo,
                                      PlanCapitalTransTypeEnum planCapitalTransTypeEnum);
    
    /**
     * 根据交易类型查询支付单
     * @param transType
     * @return
     */
    Long getActiveCntByType(PlanCapitalTransTypeEnum transType);
}