/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.plan;

import com.abssqr.plat.common.dal.mysql.auto.paging.PlanListPage;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.PriceTypeEnum;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.general.system.bpm.enums.ApproveStatusEnum;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 计划仓储
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.repo.plan: PlanRepo.java, v 0.1 2018-01-03 21:49 hongwei.wang Exp $
 */
public interface PlanRepo {

    /**
     * 分页查询获取计划列表
     *
     * @param planList
     * @return
     */
    PlanListPage getPlanList(PlanListPage planList);

    /**
     * 计划是否存在
     *
     * @param planNo
     * @return
     */
    Integer planExist(String planNo,String planName);
    
    
    /**
     * 计划 判断是否存在，不存在返回true
     *
     * @param planNo
     * @return
     */
    Integer getSamePlan(String planNo,String planName);

    /**
     * 新增计划
     *
     * @param plan
     * @return
     */
    void addPlan(PlanBase plan);

    /**
     * 修改计划
     *
     * @param plan
     */
    void updatePlan(PlanBase plan);
    
    /**
     * 修改原表审批状态
     * @param planNo
     * @param gmtModifier
     * @param approveStatus
     */
    
    void updateStatusByPlanNo(String planNo, String gmtModifier, ApproveStatusEnum approveStatus);
    
    /**
     * 修改原表计划状态
     * @param planNo
     * @param status
     * @param approveStatus
     */
    void updateStatus(String planNo,PlanStatusEnum status,ApproveStatusEnum approveStatus);
    
    /**
     * 修改原表Passed
     * @param planNo
     * @param passed
     */
    void updatePassed(String planNo,Boolean passed);
    /**
     * 获取计划
     * @param planNo
     * @return
     */
    PlanBase getPlanByNo(String planNo);
    
    /**
     * 获取所计划
     * @param planNo
     * @return
     */
    PlanBase lockPlanByNo(String planNo);

    /**
     * 查询
     * @param planNos
     * @return
     */
    <T extends PlanBase> List<T> getPlanByPlanNos(List<String> planNos,PlanTypeEnum planType, PlanStatusEnum status);
    /**
     * 查询
     * @param planNos
     * @return
     */
    <T extends PlanBase> List<T> getPlanByPlanNos(List<String> planNos,PlanTypeEnum planType, List<PlanStatusEnum> statusList);
    /**
     * 获取所有存续期的计划
     *
     * @return
     */
    <T extends PlanBase> List<T> getPlansByStatus(PlanTypeEnum planType, PlanStatusEnum status);
    
    /**
     * 获取所有存续期和准备中的计划
     *
     * @param statusList
     * @return
     */
    <T extends PlanBase> List<T> getPlansByStatusList(PlanTypeEnum planType,List<PlanStatusEnum> statusList);

    
    /**
     * 获取指定计划状态下的规模
     * @param planStatusEnum
     * @return
     */
    Long getAllPlanAmt(PlanTypeEnum planType,PlanStatusEnum planStatusEnum);
    
    /**
     * 查询计划名称
     * @param planNos
     * @return
     */
    Map<String, String> getPlanNames(List<String> planNos);

    /**
     * 获取放款计划编号
     * @param ownOrgs
     * @return
     */
    Map<String,List<String>> getPlanBySpvs(List<String> ownOrgs);
    /**
     * 获取放款计划编号
     * @param spvPlanNo
     * @return
     */
    List<String> getPlanNosBySpvNo(String spvPlanNo);

    /**
     *
     * 根据服务机构获取存续期放款计划
     * @param managerOrgCode
     * @return
     */
    List<OrganizationEntity> getSpvByManagerOrgCode(String managerOrgCode);

    /**
     * 查询每日任务失败的数量
     * @param curDate
     * @return
     */
    Long getFailDailyCount(Date curDate);

    /**
     * 更新定价类型
     * @return
     */
    Long updatePirceType(String planNo, PriceTypeEnum priceType);

    /**
     * 删除计划信息
     * @param planNo
     * @return
     */
    int deleteByPlanNo(String planNo);


    List<PlanBase> getAllPlanAstByType(String planType);
    
    /**
     * 通过map 缓存plan
     * @param planMap
     * @param planNo
     * @return
     */
    PlanBase getPlanByMapCache(Map<String, PlanBase> planMap, String planNo);
}
