/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.task.impl;

import java.util.ArrayList;
import java.util.List;

import com.abssqr.plat.common.dal.mysql.auto.dao.TaskGroupDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.TaskGroupDO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.HostTaskGroupCnt;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.convertor.TaskGroupConvert;
import com.abssqr.plat.common.model.domain.task.TaskGroup;
import com.abssqr.plat.common.model.repo.task.TaskGroupRepo;
import com.general.system.common.util.VarChecker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.repo.task.impl: TaskGroupRepoImpl.java, v 0.1 2019-12-10 11:48 bangis.wangdf Exp $ 
 */
@Component
public class TaskGroupRepoImpl implements TaskGroupRepo {
    @Autowired
    private TaskGroupDAO taskGroupDAO;

    @Override
    public void saveTaskGroups(List<TaskGroup> taskGroups){
        VarChecker.checkNotEmpty(taskGroups,"taskGroups 不能为空");
        VarChecker.checkArgument(taskGroups.size()<1000,"taskGroups 拆分不能超过1000个");
        List<TaskGroupDO> taskGroupDOS = TaskGroupConvert.model2DOS(taskGroups);
        taskGroupDAO.insertBatch(taskGroupDOS);
    }

    @Override
    public List<TaskGroup> getTaskGroup(String jobName, String hostName) {
        List<TaskGroupDO> taskGroupDO = taskGroupDAO.getExecTaskGroupByHostIpAndJobName(jobName,
            hostName);
        List<TaskGroup> taskGroups=new ArrayList<>(2);
        taskGroupDO.forEach(v->{
            taskGroups.add(TaskGroupConvert.do2Model(v));
        });
        return taskGroups;
    }

    @Override
    public void updateTaskStatus(Long id,TaskStatusEnum status,TaskStatusEnum oldStatus) {
       int cnt = taskGroupDAO.updateTaskGroupStatusById(id, TaskStatusEnum.SUCCESS.getCode());
       VarChecker.checkEquals(cnt,1,"更新失败");
    }


    @Override
    public List<TaskGroup> getUnFinishedTaskGroups(String jobName) {
        List<TaskGroupDO> unFinishTaskGroups = taskGroupDAO.getAllUnFinishTaskGroupByJobName(jobName);
        List<TaskGroup> taskGroups = TaskGroupConvert.dos2Models(unFinishTaskGroups);
        return taskGroups;
    }

    @Override
    public void markTaskGroupInit(Long id ){
       int cnt = taskGroupDAO.updateTaskGroupInit(id);
       VarChecker.checkEquals(cnt,1,"更新失败");
    }

    @Override
    public void markTaskGroupDisable(Long id) {
        int cnt = taskGroupDAO.updateTaskGroupDisable(id,true,false);
        VarChecker.checkEquals(cnt,1,"更新失败");
    }

    @Override
    public void markTaskGroupEnable(Long id) {
        int cnt = taskGroupDAO.updateTaskGroupDisable(id,false,true);
        VarChecker.checkEquals(cnt,1,"更新失败");
    }

    @Override
    public void markTaskGroupFinish(Long id) {
        int cnt = taskGroupDAO.updateTaskGroupFinish(id);
        VarChecker.checkEquals(cnt,1,"更新失败");
    }

    @Override
    public void distributeTaskGroup(Long id, String hostName) {
        int cnt = taskGroupDAO.updateTaskGroupOwner(id,hostName);
        VarChecker.checkEquals(cnt,1,"更新失败");
    }

    @Override
    public List<HostTaskGroupCnt> getHostTaskGroupCnt(String jobName) {
        List<HostTaskGroupCnt> hostTaskGroupCnt = taskGroupDAO.getHostTaskGroupCnt(jobName);
        return hostTaskGroupCnt;
    }


}
