/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.wk.repo.impl;

import com.abssqr.plat.common.dal.sync.auto.dao.WkAbsMfsBillCtrDAO;
import com.abssqr.plat.common.dal.sync.auto.dataobject.WkAbsMfsBillCtrDO;
import com.abssqr.plat.common.dal.sync.auto.resultmap.MfsBillCtrIdRange;
import com.abssqr.plat.common.dal.sync.auto.resultmap.MfsFailSyncBill;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.wk.domain.sync.WkMfsSyncCtr;
import com.abssqr.plat.common.model.wk.repo.WkMfsSyncCtrRepo;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.wk.repo.impl: WkMfsSyncCtrRepoImpl.java, v 0.1 2019-06-18 17:44 zhenxuan.luo Exp $
 */
@Component
public class WkMfsSyncCtrRepoImpl implements WkMfsSyncCtrRepo {

    @Autowired
    private WkAbsMfsBillCtrDAO wkAbsMfsBillCtrDAO;


    @Override
    @PerformTrace
    public List<WkMfsSyncCtr> selectInitTaskWithRange( Integer limit, Long startId, Long endId) {
        List<WkAbsMfsBillCtrDO> dos = wkAbsMfsBillCtrDAO.selectInitTaskByIdRange(endId,startId, limit);
        return convert(dos);
    }
    
    @Override
    public WkMfsSyncCtr lock(Long id) {
        WkAbsMfsBillCtrDO ctrDO = wkAbsMfsBillCtrDAO.lockById(id);
        return convert(ctrDO);
    }

    @Override
    public boolean isHasRemainTask(String orgCode) {
        List<WkAbsMfsBillCtrDO> dos = wkAbsMfsBillCtrDAO.selectRemainTask(orgCode);
        return !CollectionUtils.isEmpty(dos);
    }

    @Override
    @PerformTrace
    public boolean isHasRemainTask( Long minId, Long maxId) {
        List<WkAbsMfsBillCtrDO> dos = wkAbsMfsBillCtrDAO.selectRemainTaskSplit(maxId,minId);
        return !CollectionUtils.isEmpty(dos);
    }

    @Override
    public int updateTaskStatus(Long id, TaskStatusEnum curStatus, TaskStatusEnum taskStatus) {
        return wkAbsMfsBillCtrDAO.updateTaskStatus(id, curStatus.getCode(), taskStatus.getCode());
    }

    @Override
    @PerformTrace
    public int updateTaskExec(Date lastExecTime, String lastHost, List<Long> idList) {
        if(CollectionUtils.isNotEmpty(idList)&& idList.size()>1000){
            List<List<Long>> partition = Lists.partition(idList, 1000);
            for (List<Long> ids : partition) {
               int upsize = wkAbsMfsBillCtrDAO.updateTaskExec(lastExecTime, lastHost, ids);
                if(upsize!= ids.size()){
                    return upsize;
                }
            }
            return idList.size();
        }else{
          return   wkAbsMfsBillCtrDAO.updateTaskExec(lastExecTime, lastHost, idList);
        }
    }

    @Override
    public int updateTaskExep(Date nextExecTime, Long id, TaskStatusEnum taskStatus) {
        return wkAbsMfsBillCtrDAO.updateTaskExep(nextExecTime, id, taskStatus.getCode());
    }

    @Override
    public int updateHangTasksByIp(String lastHost) {
        return wkAbsMfsBillCtrDAO.updateHangTasksByIp(lastHost);
    }


    @Override
    public MfsFailSyncBill countFailSyncBill(Date syncDate, Long execTimes, List<String> list) {
        return wkAbsMfsBillCtrDAO.countFailSyncBill(syncDate, execTimes, list);
    }

    @Override
    public MfsBillCtrIdRange selectMaxIdAndMinIdOfInitTask(Long limit, Long minId) {
        return wkAbsMfsBillCtrDAO.selectMaxIdAndMinIdOfInitTask(limit,minId);
    }

    @Override
    public Long getTaskCnt() {
        return wkAbsMfsBillCtrDAO.selectInitTaskCnt();
    }

    private List<WkMfsSyncCtr> convert(List<WkAbsMfsBillCtrDO> dos) {
        if (ToolUtil.isEmpty(dos)) {
            return null;
        }
        List<WkMfsSyncCtr> res = Lists.newArrayList();
        for (WkAbsMfsBillCtrDO ctrDO : dos) {
            res.add(convert(ctrDO));
        }
        return res;
    }
    @Override
    public void deleteFailTasks(String orgCode){
        wkAbsMfsBillCtrDAO.deleteFailTasksByIgnore(orgCode);
    }

    private WkMfsSyncCtr convert(WkAbsMfsBillCtrDO ctrDO) {
        if (ctrDO == null){return null;}
        WkMfsSyncCtr wkMfsSyncCtr = new WkMfsSyncCtr();
        wkMfsSyncCtr.setId(ctrDO.getId());
        wkMfsSyncCtr.setOrgNo(ctrDO.getOrgNo());
        wkMfsSyncCtr.setLoanId(ctrDO.getLoanId());
        wkMfsSyncCtr.setProductId(ctrDO.getProductId());
        wkMfsSyncCtr.setLoanStatus(ctrDO.getLoanStatus());
        wkMfsSyncCtr.setProductName(ctrDO.getProductName());
        wkMfsSyncCtr.setAbsSyncDate(ctrDO.getAbsSyncDate());
        wkMfsSyncCtr.setTaskStatus(TaskStatusEnum.getByCode(ctrDO.getAbsStatus()));
        wkMfsSyncCtr.setNextExecTime(ctrDO.getNextExecTime());
        wkMfsSyncCtr.setLastExecTime(ctrDO.getLastExecTime());
        wkMfsSyncCtr.setLastHost(ctrDO.getLastHost());
        wkMfsSyncCtr.setExecTimes(ctrDO.getExecTimes());
        return wkMfsSyncCtr;
    }


}