/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.util.profiler;

import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.common.util.loggers.CommonLoggers;
import com.general.system.common.util.LogUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.util.profiler: PerformTraceAspect.java, v 0.1 2019-11-28 上午02:48 zhenxuan.luo Exp $ 
 */
@Aspect
@Component
public class PerformTraceAspect {
    
    private final static Logger LOGGER = CommonLoggers.PERFORM_TRACE;
    
    @Autowired
    private AbsConfig absConfig;
    
    @Pointcut("@annotation(com.abssqr.plat.common.util.profiler.PerformTrace)")
    public void performTraceAspectPointCut() {
    }
    
    @After("performTraceAspectPointCut()")
    public void after(JoinPoint joinPoint) {
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        PerformTrace trace = method.getAnnotation(PerformTrace.class);
        if (trace != null) {
            PerformRecord record = PerformTraceContext.leave();
            if (record.getParent() != null && record.getParent().isRoot()) {
                if (record.getElapse() >= absConfig.getPerformThreshold()) {
                    LogUtil.info(LOGGER, "Perform trace: {0}", record.getPerformDetail(0));
                }
            }
        }
    }
    
    @Before("performTraceAspectPointCut()")
    public void befor(JoinPoint joinPoint) {
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        PerformTrace trace = method.getAnnotation(PerformTrace.class);
        String name = joinPoint.getTarget().getClass().getSimpleName() + "." + method.getName();
        if (trace != null) {
            PerformTraceContext.enter(name);
        }
    }
}