/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */

package com.abssqr.plat.core.service.ast;

import java.util.Date;
import java.util.List;

import com.abssqr.plat.common.facade.enums.AstPackStatusEnum;
import com.abssqr.plat.common.facade.enums.AstPackTypeEnum;
import com.abssqr.plat.common.facade.enums.AstScRuleRightCondEnum;
import com.abssqr.plat.common.facade.enums.AstScreenRelTypeEnum;
import com.abssqr.plat.common.facade.enums.AstTypeEnum;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.model.domain.ast.AstPack;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.ast.AstPackRepo;
import com.abssqr.plat.common.model.repo.rule.AstScreenRuleRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.util.CommonTaskUtil;

import com.general.system.common.util.VarChecker;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author xiachenxiang
 * @version com.abssqr.plat.core.service.ast: AstPackManager.java, v 0.1 2020-04-18 4:16 PM xiachenxiang Exp $
 */
@Component
public class AstPackManager {
    @Autowired
    AstPackRepo          astPackRepository;
    @Autowired
    AstScreenRuleRepo    astScreenRuleRepo;
    @Autowired
    CommonTaskRepository commonTaskRepository;
    @Autowired
    OrgAcctDayComponent  orgAcctDayComponent;
    public static final List<AstPackStatusEnum> canPack = Lists.newArrayList(AstPackStatusEnum.SCEEN_FIS,AstPackStatusEnum.UNPACK);
    /**
     * 新增资产包
     *
     * @param astPack
     * @return
     */
    public String add(AstPack astPack) {
        // 1. 校验重复
        AstPack checkAstPack = astPackRepository.getByAbsPackPackNo(astPack.getPackNo());
        VarChecker.checkArgument(checkAstPack == null, "资产包已存在{0}", astPack.getPackNo());
        Long cnt = astPackRepository.cntByPackName(astPack.getPackName());
        VarChecker.checkEquals(cnt, 0L, "资产包名重复");
        astPackRepository.insert(astPack);

        // 3. 保存筛选规则
        astScreenRuleRepo.insertBatch(astPack.getScreenRuleList());

        // 4. 插入资产筛选任务
        CommonTask commonTask = CommonTaskUtil.buildCommonTaskWithContext(astPack.getPackNo(),CommonTaskTypeEnum.PACK_MATCH,astPack.getAstPoolNo());
        commonTaskRepository.create(commonTask);
        return astPack.getPackNo();
    }

    /**
     * 创建产品默认的资产池
     * @param astPoolNo
     * @param astTypeEnum
     */
    public void craeteDefaultPack(String astPoolNo, AstTypeEnum astTypeEnum,String prodNo) {
        AstPack astPack = new AstPack();
        astPack.setAstPoolNo(astPoolNo);
        astPack.setPackNo(astPoolNo);
        astPack.setPackType(AstPackTypeEnum.DEFAULT);
        astPack.setStatus(AstPackStatusEnum.INIT);
        astPack.setAstType(astTypeEnum);
        createDefaultScreenRule(prodNo,astPack.getPackNo());
        astPackRepository.insert(astPack);
    }

    /**
     * 获取资产包
     * @param packNo
     * @return
     */
    public AstPack get(String packNo) {
        VarChecker.checkNotEmpty(packNo,"包编号必填");
        AstPack astPack = astPackRepository.getByAbsPackPackNo(packNo);
        VarChecker.checkNotNull(astPack,"找不到对应的资产包{0}",packNo);
        List<AstScreenRule> ruleList = astScreenRuleRepo.getScreenRuleList(packNo);
        astPack.setScreenRuleList(ruleList);
        return astPack;
    }

    /**
     * 创建产品默认的筛选规则（去数据仓库筛选产品编码对应的的资产）
     * @param prodNo
     * @param packNo
     */
    //TODO 加筛选规则
    public void createDefaultScreenRule(String  prodNo,String packNo){
        AstScreenRule astScreenRule=new AstScreenRule();
        astScreenRule.setRelNo(packNo);
        astScreenRule.setRelType(AstScreenRelTypeEnum.PACK);
        astScreenRule.setRightCondition(AstScRuleRightCondEnum.eq);
        astScreenRule.setRightValue(prodNo);
        astScreenRuleRepo.insertBatch(Lists.newArrayList(astScreenRule));
    }

    /**
     * 封包
     *
     * @param packNo
     * @param sealPackDate
     */
    public void sealPack(String packNo, Date sealPackDate) {
        AstPack astPack = astPackRepository.getByAbsPackPackNo(packNo);
        VarChecker.checkNotNull(astPack,"{0}包不存在",packNo);
        VarChecker.checkArgument(canPack.contains(astPack.getStatus()),"资产包状态不满足");
        Day astAcctDay = orgAcctDayComponent.getAstAcctDay(OrgCodeEnum.ABS);
        astPackRepository.sealPack(packNo,astAcctDay.getDate(), sealPackDate);
    }
}
