package com.abssqr.plat.test;
/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

import java.util.Date;

import com.abssqr.plat.common.dal.mysql.auto.dao.AcctDateDAO;
import com.abssqr.plat.common.dal.sync.auto.dao.WkAbsMfsBillCtrDAO;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.model.domain.accounting.AcctDate;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.acct.AcctDateRepo;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;

import cn.hutool.core.date.DateUtil;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.VarChecker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockitoTestExecutionListener;
import org.springframework.test.context.TestExecutionListeners;

/**
 * 想要使用Mockito，必须添加
 *
 * @author guozhenhua
 * @version com.juhedu.pay.auto.test: com.abssqr.plat.test.BaseAutoTests.java, v 0.1 2018-04-17 14:07 hongwei.wang Exp $
 * @TestExecutionListeners(listeners = MockitoTestExecutionListener.class)'Ø
 */
@TestExecutionListeners(listeners = MockitoTestExecutionListener.class)
public class BaseAutoTests extends BaseAutoWithOutMockTests {

    @Autowired
    private WkAbsMfsBillCtrDAO wkAbsMfsBillCtrDAO;
    @Autowired
    private AcctDateDAO        acctDateDAO;

    @Autowired
    private AcctDateRepo acctDateRepo;

    /**
     * 更新会计日
     *
     * @param strDate
     */
    protected void updateDate(String strDate, String orgCode) {
        Date acctDate = DateUtil.parse(strDate);
        this.updateDate(acctDate, orgCode);
    }

    protected void updateDate(Date acctDate, String orgCode) {
        OrgCodeEnum org = OrgCodeEnum.getByCode(orgCode);
        VarChecker.checkNotNull(org, "不匹配的机构编码={0}", orgCode);
        AcctDate now = acctDateRepo.getAcctDate(org);
        if (now == null) {
            now = new AcctDate();
            now.setOrgCode(org);
            now.setAcctDate(acctDate);
            now.setEff(true);
            acctDateRepo.addAcctDate(now);
        } else if (!DateTimeUtil.isSameDay(now.getAcctDate(), acctDate)) {
            now.setAcctDate(acctDate);
            acctDateRepo.modifyAcctDate(now);
        }
    }

    @Autowired
    private JobControlRepository jobControlRepository;

    /**
     * 停止任务
     *
     * @param jobName
     * @return
     */
    protected JobControl invalidJobCtr(String jobName) {
        JobControl jobControl = jobControlRepository.selectByJobName(jobName);
        if (null == jobControl) {
            jobControl = new JobControl();
            jobControl.setJobName(jobName);
            jobControl.setIsEff(YNEnum.N);
            jobControlRepository.insert(jobControl);
        } else {
            jobControl.setIsEff(YNEnum.N);
            jobControlRepository.update(jobControl);
        }

        return jobControl;
    }

    /**
     * 开启任务
     *
     * @param jobName
     */
    protected void validJobCtr(String jobName) {
        JobControl jobControl = jobControlRepository.selectByJobName(jobName);
        if (null == jobControl) {
            jobControl = new JobControl();
            jobControl.setJobName(jobName);
            jobControl.setIsEff(YNEnum.N);
            jobControlRepository.insert(jobControl);
        }
        validJobCtr(jobControl);
    }

    protected void validJobCtr(JobControl jobControl) {
        jobControl.setIsEff(YNEnum.Y);
        jobControlRepository.update(jobControl);
    }

}
